-- NextUp
-- Written by widell

-- Changelog:
-- 1.2.4 --
--	Added Balance druid spec and cleaned up the code a little.
-- 1.2.3 --
--	Updated all priority lists for WoW 5.2.
--	Added dps cooldowns to the Shaman and Rogue priorities.
--  Fixed a minor bug in the Enhancement Shaman Priority Icon's behaivour.
--	Moved Death Strike to the Interrupt Icon for Blood Death Knights. It will now also only be recommended if the player is missing at least the 7% HP always stolen with Death Strike.
--  Added Strangulate to the Death Knight Interrupt Icon.
-- 1.2.2 --
--	Raise dead has been moved to the Reminder Icon for Death Knights. It will only be shown in combat, and for Blood only if the player does not have the Death Pact talent
-- 1.2.1 --
--	Bug fix. Elemental Blast was recommended for Enhancement Shamans even though they had chosen another lvl 90 talent.
-- 1.2.0 --
--	Major changes in code structure and how NextUp does things. For the user, the visible changes are as follows:
--	Enhancement specialization: The Priority Icon now changes much smoother and shows the next spell's icon ahead of time if no spell is ready to be used just yet.
--	Bug with the GCD calculation fixed. It made the priority icon freak out when haste pushed the GCD below the 1 sec cap.
--  Made a couple of changes to the Blood and Frost Death Knight specs as suggested by user comments on WoWinterface:
--  For Blood, Raise Dead will now only be recommended if the player did NOT select the Death Pact talent. In that case, the player will most likely want to manage that cooldown manually.
--  For Frost, Pillar of Frost has been moved to the Reminder Icon, so the player can choose not to use Pillar of Frost on trash mobs.
--  The Combo Point Indicator now shows Lightning Shield charges 3-7 instead of 1-5 for elemental shamans, since knowing when it's about to cap is more important than knowing the lower counts.
-- 1.1.6 --
--	Unholy Death Knight specialization added. Minor improvements made to Frost and Blood Death Knight priorities.
-- 1.1.5 --
--	Updates all around! Most specs' priorities have been tweaked for even better performance.
--  For level 90 characters, the global cooldown is now calculated from their haste rating, instead of using the base 1.5 seconds.
--  NextUp does now check who casted a DOT or debuff that affects the player's priority.
--  Two specs added: Blood and Frost Death Knight.
-- 1.1.4 --
--	Bug fix. Retribution priorities were not loaded properly.
-- 1.1.3 --
--	General spell casting tweak. When casting a spell with a cast time, the priority icon will now immediately switch to the next ability in the priority list, instead of after the cast finishes.
-- 1.1.2 --
--	Minor Elemental rotation tweak. The priority icon does no longer show the Lava Burst icon when it's being cast, but rather the next ability  to use.
-- 1.1.1 --
--	Elemetal Shaman spec added. Minor bug fixed that caused the GCD overlay to show even if the current spec wasn't supported.
-- 1.1.0 --
--	Major update. Added all three Rogue specs and made NextUp compatible with all WoW client languages. (hopefully)
--	Added combo point/holy power/maelstrom stack indicators. Added a GCD overlay on the pliority icon and made the
--	icon darker when the player doesn't have enough energy. Also updated the enhancement priority slightly.
-- 1.0.5 --
--	Added Feral Druid specialization, plus did some general optimization of the code.
-- 1.0.4 --
--	New Specialization added: Guardian Druid. The Faerie Swarm talent has some dodgy behavior. Avoid if possible.
-- 1.0.3 --
--	Added slash command for switching between single target and AoE priority
-- 1.0.2 --
--	Added retribution and protection paladin specs
-- 1.0.1 --
--	Added slash commands for resizing and moving the icons
-- 1.0.0 --
--	NextUp uploaded to WoWinterface

local MainFrame = CreateFrame("FRAME",nil,UIParent) -- Dummy frame used only for registering events
	-- Select events to register
    MainFrame:RegisterEvent("ADDON_LOADED")
	MainFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	MainFrame:RegisterEvent("UNIT_POWER")
	MainFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	MainFrame:RegisterEvent("UNIT_COMBO_POINTS")
	MainFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	MainFrame:RegisterEvent("PLAYER_AURAS_CHANGED")
	MainFrame:RegisterEvent("PLAYER_LOGIN")
	MainFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
	MainFrame:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
	MainFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	MainFrame:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")

function NextUpFunctions.SetDefaultSettings()
	-- Sets default icon positions and size
	NextUpSettings = {}
	NextUpSettings.IconSize = 50
	NextUpSettings.PositionHorizontal = -200
	NextUpSettings.PositionVertical = 0
	NextUpSettings.PriorityIconEnabled = true
	NextUpSettings.InterruptIconEnabled = true
	NextUpSettings.ReminderIconEnabled = true
	NextUpSettings.ComboPointFramesEnabled = true
end

local PriorityFrame = CreateFrame("Frame",nil,UIParent)
	PriorityFrame:SetFrameStrata("BACKGROUND")
	local PriorityIconTexture = PriorityFrame:CreateTexture(nil,"BACKGROUND")
	PriorityIconTexture:SetAllPoints(PriorityFrame)
	PriorityFrame.texture = PriorityIconTexture

local ResourceLowFrame = CreateFrame("Frame",nil,PriorityFrame)
	ResourceLowFrame:SetFrameStrata("BACKGROUND")
	local ResourceLowTexture = ResourceLowFrame:CreateTexture(nil,"BACKGROUND")
	ResourceLowTexture:SetAllPoints(ResourceLowFrame)
	ResourceLowFrame.texture = ResourceLowTexture
	ResourceLowTexture:SetTexture(0,0,0,0.4)

local CooldownFrame = CreateFrame("Cooldown",nil,PriorityFrame)
	CooldownFrame:SetFrameStrata("BACKGROUND")

local InterruptFrame = CreateFrame("Frame",nil,UIParent)
	InterruptFrame:SetFrameStrata("BACKGROUND")
	local InterruptIconTexture = InterruptFrame:CreateTexture(nil,"BACKGROUND")
	InterruptIconTexture:SetAllPoints(InterruptFrame)
	InterruptFrame.texture = InterruptIconTexture

local ReminderFrame = CreateFrame("Frame",nil,UIParent)
	ReminderFrame:SetFrameStrata("BACKGROUND")
	local ReminderIconTexture = ReminderFrame:CreateTexture(nil,"BACKGROUND")
	ReminderIconTexture:SetAllPoints(ReminderFrame)
	ReminderFrame.texture = ReminderIconTexture

local _,_,ComboPointTexture = GetSpellInfo(3562)
	
local ComboPointFrame1 = CreateFrame("Frame",nil,UIParent)
	ComboPointFrame1:SetFrameStrata("BACKGROUND")
	local ComboPoint1Texture = ComboPointFrame1:CreateTexture(nil,"BACKGROUND")
	ComboPoint1Texture:SetAllPoints(ComboPointFrame1)
	ComboPointFrame1.texture = ComboPoint1Texture
	ComboPoint1Texture:SetTexture(ComboPointTexture)

local ComboPointFrame2 = CreateFrame("Frame",nil,UIParent)
	ComboPointFrame2:SetFrameStrata("BACKGROUND")
	local ComboPoint2Texture = ComboPointFrame2:CreateTexture(nil,"BACKGROUND")
	ComboPoint2Texture:SetAllPoints(ComboPointFrame2)
	ComboPointFrame2.texture = ComboPoint2Texture
	ComboPoint2Texture:SetTexture(ComboPointTexture)
	
local ComboPointFrame3 = CreateFrame("Frame",nil,UIParent)
	ComboPointFrame3:SetFrameStrata("BACKGROUND")
	local ComboPoint3Texture = ComboPointFrame3:CreateTexture(nil,"BACKGROUND")
	ComboPoint3Texture:SetAllPoints(ComboPointFrame3)
	ComboPointFrame3.texture = ComboPoint3Texture
	ComboPoint3Texture:SetTexture(ComboPointTexture)
	
local ComboPointFrame4 = CreateFrame("Frame",nil,UIParent)
	ComboPointFrame4:SetFrameStrata("BACKGROUND")
	local ComboPoint4Texture = ComboPointFrame4:CreateTexture(nil,"BACKGROUND")
	ComboPoint4Texture:SetAllPoints(ComboPointFrame4)
	ComboPointFrame4.texture = ComboPoint4Texture
	ComboPoint4Texture:SetTexture(ComboPointTexture)
	
local ComboPointFrame5 = CreateFrame("Frame",nil,UIParent)
	ComboPointFrame5:SetFrameStrata("BACKGROUND")
	local ComboPoint5Texture = ComboPointFrame5:CreateTexture(nil,"BACKGROUND")
	ComboPoint5Texture:SetAllPoints(ComboPointFrame5)
	ComboPointFrame5.texture = ComboPoint5Texture
	ComboPoint5Texture:SetTexture(ComboPointTexture)
	
local function UpdateIconProperties()
	-- Sets the size and position of the icons
	PriorityFrame:SetWidth(NextUpSettings.IconSize)
	PriorityFrame:SetHeight(NextUpSettings.IconSize)
	ResourceLowFrame:SetWidth(NextUpSettings.IconSize)
	ResourceLowFrame:SetHeight(NextUpSettings.IconSize)
	CooldownFrame:SetWidth(NextUpSettings.IconSize)
	CooldownFrame:SetHeight(NextUpSettings.IconSize)
	InterruptFrame:SetWidth(NextUpSettings.IconSize)
	InterruptFrame:SetHeight(NextUpSettings.IconSize)
	ReminderFrame:SetWidth(NextUpSettings.IconSize)
	ReminderFrame:SetHeight(NextUpSettings.IconSize)
	ComboPointFrame1:SetWidth(NextUpSettings.IconSize*(2/14))
	ComboPointFrame1:SetHeight(NextUpSettings.IconSize*(2/14))
	ComboPointFrame2:SetWidth(NextUpSettings.IconSize*(2/14))
	ComboPointFrame2:SetHeight(NextUpSettings.IconSize*(2/14))
	ComboPointFrame3:SetWidth(NextUpSettings.IconSize*(2/14))
	ComboPointFrame3:SetHeight(NextUpSettings.IconSize*(2/14))
	ComboPointFrame4:SetWidth(NextUpSettings.IconSize*(2/14))
	ComboPointFrame4:SetHeight(NextUpSettings.IconSize*(2/14))
	ComboPointFrame5:SetWidth(NextUpSettings.IconSize*(2/14))
	ComboPointFrame5:SetHeight(NextUpSettings.IconSize*(2/14))
	
	PriorityFrame:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal,NextUpSettings.PositionVertical)
	ResourceLowFrame:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal,NextUpSettings.PositionVertical)
	CooldownFrame:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal,NextUpSettings.PositionVertical)
	InterruptFrame:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal+NextUpSettings.IconSize,NextUpSettings.PositionVertical)
	ReminderFrame:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal-NextUpSettings.IconSize,NextUpSettings.PositionVertical)
	ComboPointFrame1:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal-(NextUpSettings.IconSize*(6/14)),NextUpSettings.PositionVertical-(NextUpSettings.IconSize*(9/14)))
	ComboPointFrame2:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal-(NextUpSettings.IconSize*(3/14)),NextUpSettings.PositionVertical-(NextUpSettings.IconSize*(9/14)))
	ComboPointFrame3:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal,NextUpSettings.PositionVertical-(NextUpSettings.IconSize*(9/14)))
	ComboPointFrame4:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal+(NextUpSettings.IconSize*(3/14)),NextUpSettings.PositionVertical-(NextUpSettings.IconSize*(9/14)))
	ComboPointFrame5:SetPoint("CENTER","UIParent","CENTER",NextUpSettings.PositionHorizontal+(NextUpSettings.IconSize*(6/14)),NextUpSettings.PositionVertical-(NextUpSettings.IconSize*(9/14)))
end

local PriorityIcon
local InterruptIcon
local ReminderIcon
local ComboPoints = 0
local ResourceLow = false
local SingleTargetPriority = true

-- Get player's class and a spell available <= level 10 without a cooldown. It is then used for the cooldown overlay on the priority icon
local _,PlayerClass = UnitClass("player")
local CooldownCheckSkill
if PlayerClass == "DEATHKNIGHT"	then CooldownCheckSkill = 47541		-- Death Coil
elseif PlayerClass == "DRUID"		then CooldownCheckSkill = 5176		-- Wrath
elseif PlayerClass == "MAGE"		then CooldownCheckSkill = 1449		-- Arcane Explosion (note to self: Not available until level 18)
elseif PlayerClass == "MAGE"		then CooldownCheckSkill = 44614		-- Frostfire Bolt
elseif PlayerClass == "MONK"		then CooldownCheckSkill = 100780	-- Jab
elseif PlayerClass == "PALADIN"		then CooldownCheckSkill = 105361	-- Seal of Command
elseif PlayerClass == "PRIEST"		then CooldownCheckSkill = 585		-- Smite
elseif PlayerClass == "ROGUE"		then CooldownCheckSkill = 1752		-- Sinister Strike
elseif PlayerClass == "SHAMAN"		then CooldownCheckSkill = 403		-- Lightning Bolt
elseif PlayerClass == "WARLOCK"		then CooldownCheckSkill = 686		-- Shadow Bolt
elseif PlayerClass == "WARRIOR"		then CooldownCheckSkill = 122475	-- Throw
end

local PlayerSpec
local PlayerSpecSupported
local function DeterminePlayerSpec()
	-- Determine player's current spec by looking for its signature spell in the player's spellbook
	if GetSpellInfo(GetSpellInfo(50034))		then PlayerSpec = "Blood Death Knight"		-- Blood Rites
	elseif GetSpellInfo(GetSpellInfo(49143))	then PlayerSpec = "Frost Death Knight"		-- Frost Strike
	elseif GetSpellInfo(GetSpellInfo(56835))	then PlayerSpec = "Unholy Death Knight"		-- Reaping
	elseif GetSpellInfo(GetSpellInfo(79577))	then PlayerSpec = "Balance Druid"			-- Eclipse
	elseif GetSpellInfo(GetSpellInfo(5217))		then PlayerSpec = "Feral Druid"				-- Tiger's Fury
	elseif GetSpellInfo(GetSpellInfo(62606))	then PlayerSpec = "Guardian Druid"			-- Savage Defense
	elseif GetSpellInfo(GetSpellInfo(18562))	then PlayerSpec = "Restoration Druid"		-- Swiftmend
	elseif GetSpellInfo(GetSpellInfo(34026))	then PlayerSpec = "Beast Mastery Hunter"	-- Kill Command
	elseif GetSpellInfo(GetSpellInfo(19434))	then PlayerSpec = "Marksmanship Hunter"		-- Aimed Shot
	elseif GetSpellInfo(GetSpellInfo(53301))	then PlayerSpec = "Survival Hunter"			-- Explosive Shot
	elseif GetSpellInfo(GetSpellInfo(30451))	then PlayerSpec = "Arcane Mage"				-- Arcane Blast
	elseif GetSpellInfo(GetSpellInfo(11366))	then PlayerSpec = "Fire Mage"				-- Pyroblast
	elseif GetSpellInfo(GetSpellInfo(31687))	then PlayerSpec = "Frost Mage"				-- Summon Water Elemental
	elseif GetSpellInfo(GetSpellInfo(115180))	then PlayerSpec = "Brewmaster Monk"			-- Dizzying Haze
	elseif GetSpellInfo(GetSpellInfo(115175))	then PlayerSpec = "Mistweaver Monk"			-- Soothing Mist
	elseif GetSpellInfo(GetSpellInfo(113656))	then PlayerSpec = "Windwalker Monk"			-- Fists of Fury
	elseif GetSpellInfo(GetSpellInfo(20473))	then PlayerSpec = "Holy Paladin"			-- Holy Shock
	elseif GetSpellInfo(GetSpellInfo(31935))	then PlayerSpec = "Protection Paladin"		-- Avenger's Shield
	elseif GetSpellInfo(GetSpellInfo(85256))	then PlayerSpec = "Retribution Paladin"		-- Templar's Verdict
	elseif GetSpellInfo(GetSpellInfo(47540))	then PlayerSpec = "Discipline Priest"		-- Penance
	elseif GetSpellInfo(GetSpellInfo(88625))	then PlayerSpec = "Holy Priest"				-- Holy Word: Chastice
	elseif GetSpellInfo(GetSpellInfo(15407))	then PlayerSpec = "Shadow Priest"			-- Mind Flay
	elseif GetSpellInfo(GetSpellInfo(1329))		then PlayerSpec = "Assassination Rogue"		-- Mutilate
	elseif GetSpellInfo(GetSpellInfo(13877))	then PlayerSpec = "Combat Rogue"			-- Blade Flurry
	elseif GetSpellInfo(GetSpellInfo(16511))	then PlayerSpec = "Subtelty Rogue"			-- Hemorrhage
	elseif GetSpellInfo(GetSpellInfo(51490))	then PlayerSpec = "Elemental Shaman"		-- Thunderstorm
	elseif GetSpellInfo(GetSpellInfo(60103))	then PlayerSpec = "Enhancement Shaman"		-- Lava Lash
	elseif GetSpellInfo(GetSpellInfo(61295))	then PlayerSpec = "Restoration Shaman"		-- Riptide
	elseif GetSpellInfo(GetSpellInfo(30108))	then PlayerSpec = "Affliction Warlock"		-- Unstable Affliction
	elseif GetSpellInfo(GetSpellInfo(103958))	then PlayerSpec = "Demenology Warlock"		-- Metamorphosis
	elseif GetSpellInfo(GetSpellInfo(17962))	then PlayerSpec = "Destruction Warlock"		-- Conflagrate
	elseif GetSpellInfo(GetSpellInfo(12294))	then PlayerSpec = "Arms Warrior"			-- Mortal Strike
	elseif GetSpellInfo(GetSpellInfo(23881))	then PlayerSpec = "Fury Warrior"			-- Bloodlust
	elseif GetSpellInfo(GetSpellInfo(23922))	then PlayerSpec = "Protection Warrior"		-- Shield Slam
	end

	-- Print detected spec
	if PlayerSpec then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 " .. PlayerSpec .. " spec detected.")
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 No spec detected.")
	end
		
	PlayerSpecSupported = true

	if (PlayerSpec ~= "Blood Death Knight"
	and PlayerSpec ~= "Frost Death Knight"
	and PlayerSpec ~= "Unholy Death Knight"
	and PlayerSpec ~= "Balance Druid"
	and PlayerSpec ~= "Feral Druid"
	and PlayerSpec ~= "Guardian Druid"
--	and PlayerSpec ~= "Beastmastery Hunter"
--	and PlayerSpec ~= "Marksmanship Hunter"
--	and PlayerSpec ~= "Survival Hunter"
--	and PlayerSpec ~= "Arcane Mage"
--	and PlayerSpec ~= "Fire Mage"
--	and PlayerSpec ~= "Frost Mage"
--	and PlayerSpec ~= "Brewmaster Monk"
--	and PlayerSpec ~= "Windwalker Monk"
	and PlayerSpec ~= "Protection Paladin"
	and PlayerSpec ~= "Retribution Paladin"
--	and PlayerSpec ~= "Shadow Priest"
	and PlayerSpec ~= "Assassination Rogue"
	and PlayerSpec ~= "Combat Rogue"
	and PlayerSpec ~= "Subtelty Rogue"
	and PlayerSpec ~= "Elemental Shaman"
	and PlayerSpec ~= "Enhancement Shaman"
--	and PlayerSpec ~= "Affliction Warlock"
--	and PlayerSpec ~= "Demonology Warlock"
--	and PlayerSpec ~= "Destruction Warlock"
--	and PlayerSpec ~= "Arms Warrior"
--	and PlayerSpec ~= "Fury Warrior"
--	and PlayerSpec ~= "Protection Warrior"
	) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Your current spec is not supported in this version of NextUp.")
		PlayerSpecSupported = false
	end
end

local function UpdateFrames()
	
	-- Get ability icons from the priority functions (found in separate .lua files)
	if (PlayerSpec == "Blood Death Knight") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DeathKnightBloodPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Frost Death Knight") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DeathKnightFrostPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Unholy Death Knight") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DeathKnightUnholyPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Balance Druid") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DruidBalancePriority(SingleTargetPriority)
	elseif (PlayerSpec == "Feral Druid") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DruidFeralPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Guardian Druid") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.DruidGuardianPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Beastmastery Hunter") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.HunterBeastMasteryPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Masrksmanship Hunter") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.HunterMarksmanshipPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Survival Hunter") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.HunterSurvivalPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Arcane Mage") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.MageArcanePriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Fire Mage") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.MageFirePriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Frost Mage") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.MageFrostPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Brewmaster Monk") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.MonkBrewmasterPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Windwalker Monk") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.MonkWindwalkerPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Protection Paladin") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.PaladinProtectionPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Retribution Paladin") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.PaladinRetributionPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Shadow Priest") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.PriestShadowPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Assassination Rogue") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.RogueAssassinationPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Combat Rogue") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.RogueCombatPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Subtelty Rogue") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.RogueSubteltyPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Elemental Shaman") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.ShamanElementalPriority(SingleTargetPriority)
	elseif (PlayerSpec == "Enhancement Shaman") then
		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.ShamanEnhancementPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Affliction Warlock") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarlockAfflictionPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Demonology Warlock") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarlockDemonologyPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Destruction Warlock") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarlockDestructionPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Arms Warrior") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarriorArmsPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Fury Warrior") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarriorFuryPriority(SingleTargetPriority)
--	elseif (PlayerSpec == "Protection Warrior") then
--		PriorityIcon,InterruptIcon,ReminderIcon,ComboPoints,ResourceLow = NextUpFunctions.WarriorProtectionPriority(SingleTargetPriority)
	else
		-- Clear icons if current spec is not supported
		PriorityIcon,InterruptIcon,ReminderIcon = nil
	end
	
	-- Update icons if enabled
	if NextUpSettings.PriorityIconEnabled then
		PriorityIconTexture:SetTexture(PriorityIcon)
		PriorityFrame:Show()
	else
		PriorityFrame:Hide()
	end

	if (PlayerSpecSupported and NextUpFunctions.TargetIsEnemy()) then
		CooldownFrame:Show()
	else
		CooldownFrame:Hide()
	end
	
	-- If player is in combat, but there is  no ability recommended, show a semi-transparent black square
	if (not PriorityIcon and PlayerSpecSupported and NextUpFunctions.TargetIsEnemy()) then
		PriorityIconTexture:SetTexture(0,0,0,0.5)
	end

	if ResourceLow then
		ResourceLowFrame:Show()
	else
		ResourceLowFrame:Hide()
	end
	
	if NextUpSettings.InterruptIconEnabled then
		InterruptIconTexture:SetTexture(InterruptIcon)
		InterruptFrame:Show()
	else
		InterruptFrame:Hide()
	end
	
	if NextUpSettings.ReminderIconEnabled then
		ReminderIconTexture:SetTexture(ReminderIcon)
		ReminderFrame:Show()
	else
		ReminderFrame:Hide()
	end

	if NextUpSettings.ComboPointFramesEnabled and ComboPoints then
		if (ComboPoints == 1) then
			ComboPointFrame1:Show()
			ComboPointFrame2:Hide()
			ComboPointFrame3:Hide()
			ComboPointFrame4:Hide()
			ComboPointFrame5:Hide()
		elseif (ComboPoints == 2) then
			ComboPointFrame1:Show()
			ComboPointFrame2:Show()
			ComboPointFrame3:Hide()
			ComboPointFrame4:Hide()
			ComboPointFrame5:Hide()
		elseif (ComboPoints == 3) then
			ComboPointFrame1:Show()
			ComboPointFrame2:Show()
			ComboPointFrame3:Show()
			ComboPointFrame4:Hide()
			ComboPointFrame5:Hide()
		elseif (ComboPoints == 4) then
			ComboPointFrame1:Show()
			ComboPointFrame2:Show()
			ComboPointFrame3:Show()
			ComboPointFrame4:Show()
			ComboPointFrame5:Hide()
		elseif (ComboPoints >= 5) then
			ComboPointFrame1:Show()
			ComboPointFrame2:Show()
			ComboPointFrame3:Show()
			ComboPointFrame4:Show()
			ComboPointFrame5:Show()
		else
			ComboPointFrame1:Hide()
			ComboPointFrame2:Hide()
			ComboPointFrame3:Hide()
			ComboPointFrame4:Hide()
			ComboPointFrame5:Hide()
		end
	end
	
end

local function HideFrames()
		PriorityFrame:Hide()
		InterruptFrame:Hide()
		ReminderFrame:Hide()
		ComboPointFrame1:Hide()
		ComboPointFrame2:Hide()
		ComboPointFrame3:Hide()
		ComboPointFrame4:Hide()
		ComboPointFrame5:Hide()
end

function NextUpFunctions.OnEvent(self, Event, arg1)

	if (Event == "ADDON_LOADED" and arg1 == "NextUp") then
		-- Check if this is the first time NextUp has been loaded (per character) and set default settings if so
        if (not NextUpSettings) then
			NextUpFunctions.SetDefaultSettings()
		end
		UpdateIconProperties()

	elseif Event == "ACTIONBAR_UPDATE_COOLDOWN" then
		if ((GetSpellCooldown(CooldownCheckSkill) ~= 0) and NextUpFunctions.TargetIsEnemy()) then
			CooldownFrame:SetCooldown(GetSpellCooldown(CooldownCheckSkill))
		end

	elseif(PlayerSpecSupported and (Event == "COMBAT_LOG_EVENT_UNFILTERED" or Event == "PLAYER_REGEN_DISABLED" or Event == "UNIT_COMBO_POINTS" or Event == "PLAYER_AURAS_CHANGED" or Event == "UNIT_POWER" or Event == "PLAYER_TARGET_CHANGED")) then
		UpdateFrames()
		
	-- Determine and print out a confirmation of the player's current spec when they login
	elseif(Event == "PLAYER_LOGIN") then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 :: NextUp loaded ::")
		HideFrames()
		DeterminePlayerSpec()

	-- Determine and print out a confirmation of the player's current spec when they change specs
	elseif(Event == "ACTIVE_TALENT_GROUP_CHANGED") then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 :: NextUp ::")
		DeterminePlayerSpec()
		UpdateFrames()

	end
end

local function TogglePriorityFrame()
	if NextUpSettings.PriorityFrameEnabled then
		NextUpSettings.PriorityFrameEnabled = false
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Priority icon is now disabled")
		PriorityFrame:Hide()
	else
		NextUpSettings.PriorityFrameEnabled = true
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Priority icon is now enabled")
	end
end

local function ToggleInterruptFrame()
	if NextUpSettings.InterruptFrameEnabled then
		NextUpSettings.InterruptFrameEnabled = false
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Interrupt icon is now disabled")
		InterruptFrame:Hide()
	else
		NextUpSettings.InterruptFrameEnabled = true
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Interrupt icon is now enabled")
	end
end

local function ToggleReminderFrame()
	if NextUpSettings.ReminderFrameEnabled then
		NextUpSettings.ReminderFrameEnabled = false
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Reminder icon is now disabled")
		ReminderFrame:Hide()
	else
		NextUpSettings.ReminderFrameEnabled = true
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Reminder icon is now enabled")
	end
end

local function ToggleComboPointFrames()
	if NextUpSettings.ComboPointFramesEnabled then
		NextUpSettings.ComboPointFramesEnabled = false
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Combo point indicators are now disabled")
		ComboPointFrame1:Hide()
		ComboPointFrame2:Hide()
		ComboPointFrame3:Hide()
		ComboPointFrame4:Hide()
		ComboPointFrame5:Hide()
	else
		NextUpSettings.ComboPointFramesEnabled = true
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 Combo point indicators are now enabled")
	end
end

local function ToggleSingleTargetPriority()
	if SingleTargetPriority then
		SingleTargetPriority = false
	else
		SingleTargetPriority = true
	end
end

local function SplitString(String)
	if string.find(String, " ") then
		Command = string.sub(String, 1, string.find(String, " ")-1)
		String = string.sub(String, string.find(String, " ")+1)
		return Command, String
	else
		return String
	end
end

local function InvalidCommand(Command, String)
	if String then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 /nextup " .. Command .. " " .. String .. " is an invalid command. Type '/nextup help' for a list of commands.")
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 /nextup " .. Command .. " is an invalid command. Type '/nextup help' for a list of commands.")
	end
end

local function SlashHandler(String)

	-- Make slash command all lower case, since the if sentences are case sensitive
	String = string.lower(String)
	
	local Command
	Command, String = SplitString(String)

	if Command == "toggle" and String == "aoe" then
			ToggleSingleTargetPriority()
	else
	
		DEFAULT_CHAT_FRAME:AddMessage(" ")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55  :: NextUp ::")

		-- Iconsize
		if Command == "iconsize" and String then
			if not string.match(String, "[%a]") then
				NextUpSettings.IconSize = String
				DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55Icon Size set to " .. NextUpSettings.IconSize)
				UpdateIconProperties()
			else
				InvalidCommand(Command, String)
			end

		-- Vertical position
		elseif Command == "vposition" and String then
			if not string.match(String, "[%a]") then
				NextUpSettings.PositionVertical = String
				DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55Vertical position set to " .. NextUpSettings.PositionVertical)
				UpdateIconProperties()
			else
				InvalidCommand(Command, String)
			end
			
		-- Horizontal position
		elseif Command == "hposition" and String then
			if not string.match(String, "[%a]") then
				NextUpSettings.PositionHorizontal = String
				DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55Horizontal position set to " .. NextUpSettings.PositionHorizontal)
				UpdateIconProperties()
			else
				InvalidCommand(Command, String)
			end
			
		-- Toggles
		elseif Command == "toggle" and String then
			if String == "priority" then
				TogglePriorityFrame()
			elseif String == "interrupt" then
				ToggleInterruptFrame()
			elseif String == "reminder" then
				ToggleReminderFrame()
			elseif String == "combopoints" then
				ToggleComboPointFrames()
			else
				InvalidCommand(Command, String)
			end
			
		-- Reset all settings
		elseif Command == "reset" then
			NextUpFunctions.SetDefaultSettings()
			UpdateIconProperties()
			DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 NextUp has been reset")

		-- Print slash command list
		elseif Command == "help" or Command == "" or Command == nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff Slash commands:")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff help |cffffcc55shows this list.")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff reset |cffffcc55resets all settings to defaults.")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff toggle priority |cffffcc55toggles the priority icon")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff toggle interrupt |cffffcc55toggles the interrupt icon")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff toggle reminder |cffffcc55toggles the reminder icon")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff toggle combopoints |cffffcc55toggles the combo point indicators")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff toggle aoe |cffffcc55switches between the single target and the AoE priority list.")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff iconsize <size> |cffffcc55sets the icon size in pixels.")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff hposition <offset> |cffffcc55and |cffffffffvposition <offset> |cffffcc55sets")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffcc55 offset from the center of the screen in pixels")
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff Tip: Macro '/nextup toggle aoe' to an actionbar button for maximum ease of use.")
		else
			InvalidCommand(Command, String)
		end
	end
end

MainFrame:SetScript("OnEvent", NextUpFunctions.OnEvent)

SLASH_NEXTUP1 = "/nextup"
SLASH_NEXTUP2 = "/nu"
SlashCmdList["NEXTUP"] = SlashHandler