function NextUpFunctions.DeathKnightBloodPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(47541) -- Get Death Coil CD
	
	-- Get cooldowns
	local EmpowerRuneWeaponCD = NextUpFunctions.GetSpellCooldown(47568)
	local DeathAndDecayCD = NextUpFunctions.GetSpellCooldown(43265)
	local UnholyBlightCD = NextUpFunctions.GetSpellCooldown(115989)
	local HornOfWinterCD = NextUpFunctions.GetSpellCooldown(57330)
	local PlagueLeechCD = NextUpFunctions.GetSpellCooldown(123693)
	local StrangulateCD = NextUpFunctions.GetSpellCooldown(47476)
	local MindFreezeCD = NextUpFunctions.GetSpellCooldown(47528)
	local RaiseDeadCD = NextUpFunctions.GetSpellCooldown(46584)
	local OutbreakCD = NextUpFunctions.GetSpellCooldown(77575)

	-- Get buffs
	local _,_,_,BloodCharges = UnitBuff("player", GetSpellInfo(114851))
	if not BloodCharges then BloodCharges = 0 end
	
	local RunicCorruptionUp = UnitBuff("player", GetSpellInfo(51462))
	local CrimsonScourgeUp = UnitBuff("player", GetSpellInfo(81136))
	local BloodPresenceUp = UnitBuff("player", GetSpellInfo(48263))

	local DarkSuccorUp
	local DarkSuccorRemaining = 0
	local _,_,_,_,_,_,DarkSuccorExpirationTime = UnitBuff("player", GetSpellInfo(101568))
	if DarkSuccorExpirationTime then
		DarkSuccorUp = true
		DarkSuccorRemaining = (DarkSuccorExpirationTime - CurrentTime)
	end
	
	local FrostFeverUp
	local FrostFeverRemaining = 0
	local _,_,_,_,_,_,FrostFeverExpirationTime,FrostFeverCaster = UnitDebuff("target", GetSpellInfo(55095))
	if (FrostFeverCaster == "player") then
		FrostFeverUp = true
		FrostFeverRemaining = (FrostFeverExpirationTime - CurrentTime)
	end
	
	local BloodPlagueUp
	local BloodPlagueRemaining = 0
	local _,_,_,_,_,_,BloodPlagueExpirationTime,BloodPlagueCaster = UnitDebuff("target", GetSpellInfo(55078))
	if (BloodPlagueCaster == "player") then
		BloodPlagueUp = true
		BloodPlagueRemaining = (BloodPlagueExpirationTime - CurrentTime)
	end
	
	-- Get power levels
	local RunicPower = UnitPower("player", 6)

	local BloodRunes = 0
	local FrostRunes = 0
	local UnholyRunes = 0
	local DeathRunes = 0
	
	local Runes = {}
	
	-- Returns true if the rune in question is ready to be used within one GCD
	local function RuneReady(i)
		if ((CurrentTime - GetRuneCooldown(i)) >= ((select(2,GetRuneCooldown(i))) - GCD)) then
			return true
		else
			return false
		end
	end

	-- Cycles through the 6 runes using the above function to determine how many of each kind are ready to use next GCD.
	for i=1,6,1 do
		Runes[i] = RuneReady(i)
		if (RuneReady(i) and (GetRuneType(i) == 1)) then BloodRunes = BloodRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 2)) then UnholyRunes = UnholyRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 3)) then FrostRunes = FrostRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 4)) then DeathRunes = DeathRunes+1 end
	end

	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Blood tap if the player has more than 10 charges and either A: is about to cap Runic Power or B: has a Killing Machine Proc
			if(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and ((RunicPower > 76) or KillingMachineUp) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Remove diseases with Plague Leech if one is about to fall off or if Outbreak is just about to come off cooldown so both can be applied right after Leech
			elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3) or (GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= (2*GCD)))) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(123693)
			
			-- Apply diseases with outbreak if one is about to fall off or one is missing
			elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(77575)
			
			-- Unholy Blight if chosen as tier 1 talent and one disease is about to fall off
			elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(115989)

			-- Blood Boil to refresh diseases if one is about to fall off and Scarlet Fever has been learned
			elseif(GetSpellInfo(GetSpellInfo(48721)) and GetSpellInfo(GetSpellInfo(81132)) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3)) and (((FrostRunes + DeathRunes) >= 1) or CrimsonScourgeUp)) then
				_,_,PriorityIcon = GetSpellInfo(48721)

			-- Apply Blood Plague with Plague Strike
			elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45462)

			-- Apply Frost fever with Icy Touch
			elseif((GetSpellInfo(GetSpellInfo(45477))) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45477)
				
			-- Soul Reaper if target hp is below 35 percent
			elseif(GetSpellInfo(GetSpellInfo(130735)) and (TargetHPPercent <= 35) and ((FrostRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(130735)

			-- Rune Strike or Death Coil to avoid getting runic power capped
			elseif(RunicPower > 88) then
				if (GetSpellInfo(GetSpellInfo(56815))) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				else
					_,_,PriorityIcon = GetSpellInfo(47541)
				end

			-- Spend Blood runes on Heart Strike (Not death runes. They are saved for Death Strike)
			elseif(GetSpellInfo(GetSpellInfo(55050)) and (BloodRunes >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(55050)
				
			-- Spend Crimson Scourge procs on Death and Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and CrimsonScourgeUp and (DeathAndDecayCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(43265)
				
			-- Spend Crimson Scourge procs on Blood Boil
			elseif(GetSpellInfo(GetSpellInfo(48721)) and CrimsonScourgeUp) then
				_,_,PriorityIcon = GetSpellInfo(48721)
				
			-- Horn of Winter
			elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(57330)

			-- Rune Strike / Death Coil if player has the Runic Empowerment talent and there is a fully depleted rune
			elseif(GetSpellInfo(GetSpellInfo(81229)) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				if (GetSpellInfo(GetSpellInfo(56815)) and (RunicPower >= 30)) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Blood tap if the player has 8 or more charges
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges >= 8) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Rune Strike / Death Coil if player has the Runic Corruption talent and its buff isn't currently up
			elseif(GetSpellInfo(GetSpellInfo(51462)) and not RunicCorruptionUp) then
				if (GetSpellInfo(GetSpellInfo(56815)) and (RunicPower >= 30)) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Death And Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(43265)

			-- Rune Strike / Death coil
			elseif(RunicPower >= 30) then
				if (GetSpellInfo(GetSpellInfo(56815))) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Empower Rune Weapon to fill up runes and Runic Power
			elseif(GetSpellInfo(GetSpellInfo(47568) and (EmpowerRuneWeaponCD <= GCD))) then
				_,_,PriorityIcon = GetSpellInfo(47568)
				
			end

		else

			-- AoE priority --

			-- Blood tap if the player has more than 10 charges and either A: is about to cap Runic Power or B: has a Killing Machine Proc
			if(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and ((RunicPower > 76) or KillingMachineUp) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Remove diseases with Plague Leech if one is about to fall off or if Outbreak is just about to come off cooldown so both can be applied right after Leech
			elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3) or (GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= (2*GCD)))) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(123693)
			
			-- Apply diseases with outbreak if one is about to fall off or one is missing
			elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(77575)
			
			-- Unholy Blight if chosen as tier 1 talent and one disease is about to fall off
			elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(115989)

			-- Blood Boil to refresh diseases if one is about to fall off and Scarlet Fever has been learned
			elseif(GetSpellInfo(GetSpellInfo(48721)) and GetSpellInfo(GetSpellInfo(81132)) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3)) and (((FrostRunes + DeathRunes) >= 1) or CrimsonScourgeUp)) then
				_,_,PriorityIcon = GetSpellInfo(48721)

			-- Apply Blood Plague with Plague Strike
			elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45462)

			-- Apply Frost fever with Icy Touch
			elseif((GetSpellInfo(GetSpellInfo(45477))) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45477)
				
			-- Soul Reaper if target hp is below 35 percent
			elseif(GetSpellInfo(GetSpellInfo(130735)) and (TargetHPPercent <= 35) and ((FrostRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(130735)

			-- Rune Strike or Death Coil to avoid getting runic power capped
			elseif(RunicPower > 88) then
				if (GetSpellInfo(GetSpellInfo(47541))) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				else
					_,_,PriorityIcon = GetSpellInfo(47541)
				end

			-- Spend Blood Runes on Blood Boil
			elseif(GetSpellInfo(GetSpellInfo(48721)) and (BloodRunes >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(48721)
				
			-- Spend Crimson Scourge procs on Death and Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and CrimsonScourgeUp and (DeathAndDecayCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(43265)
				
			-- Horn of Winter
			elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(57330)

			-- Death Coil if player has the Runic Empowerment talent and there is a fully depleted rune
			elseif(GetSpellInfo(GetSpellInfo(81229)) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				if (GetSpellInfo(GetSpellInfo(56815)) and (RunicPower >= 30)) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Blood tap if the player has 8 or more charges
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges >= 8) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Rune Strike / Death Coil if player has the Runic Corruption talent its buff isn't currently up
			elseif(GetSpellInfo(GetSpellInfo(51462)) and not RunicCorruptionUp) then
				if (GetSpellInfo(GetSpellInfo(56815)) and (RunicPower >= 30)) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Death And Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(43265)

			-- Rune Strike / Death coil
			elseif(RunicPower >= 30) then
				if (GetSpellInfo(GetSpellInfo(56815))) then
					_,_,PriorityIcon = GetSpellInfo(56815)
				elseif (RunicPower >= 40) then
					_,_,PriorityIcon = GetSpellInfo(47541)
				end
				
			-- Empower Rune Weapon to fill up runes and Runic Power
			elseif(GetSpellInfo(GetSpellInfo(47568) and (EmpowerRuneWeaponCD <= GCD))) then
				_,_,PriorityIcon = GetSpellInfo(47568)
				
			end

		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Mind Freeze
		if (GetSpellInfo(GetSpellInfo(47528)) and (MindFreezeCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47528)
		
		-- Strangulate
		elseif (GetSpellInfo(GetSpellInfo(47476)) and (StrangulateCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47476)

		-- Spend Frost, Unholy and Death runes on Death Strike (only recommended if any of the following is true: Player is missing at least the 7% always stolen, player has the Dark Succor buff up and is missing at least 20% or the player's Dark Succor buff is about to run out.
		elseif(GetSpellInfo(GetSpellInfo(49998)) and ((NextUpFunctions.GetPlayerHPPercent() < 93) and not DarkSuccorUp and (((FrostRunes >= 1) and (UnholyRunes >= 1)) or ((FrostRunes >= 1) and (DeathRunes >= 1)) or ((UnholyRunes >= 1) and (DeathRunes >= 1)) or (DeathRunes >= 2))) or (DarkSuccorUp and (NextUpFunctions.GetPlayerHPPercent() <= 80)) or (DarkSuccorUp and DarkSuccorRemaining < 2)) then
			_,_,InterruptIcon = GetSpellInfo(49998)

		end
	end
	
------------------
-- ReminderIcon --
------------------

	-- Blood Presence
	if (GetSpellInfo(GetSpellInfo(48263)) and not BloodPresenceUp) then
		_,_,ReminderIcon = GetSpellInfo(48263)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then
		
		-- Raise Dead if Player does not have the Death Pact talent, in which case it Raise Dead will most likely be used for that
		if(GetSpellInfo(GetSpellInfo(46584)) and not GetSpellInfo(GetSpellInfo(48743)) and (RaiseDeadCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(46584)

		end
		
	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon, 0
end