function NextUpFunctions.DeathKnightFrostPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(47541) -- Get Death Coil CD

	-- Get cooldowns
	local EmpowerRuneWeaponCD = NextUpFunctions.GetSpellCooldown(47568)
	local PillarOfFrostCD = NextUpFunctions.GetSpellCooldown(51271)
	local DeathAndDecayCD = NextUpFunctions.GetSpellCooldown(43265)
	local UnholyBlightCD = NextUpFunctions.GetSpellCooldown(115989)
	local HornOfWinterCD = NextUpFunctions.GetSpellCooldown(57330)
	local PlagueLeechCD = NextUpFunctions.GetSpellCooldown(123693)
	local StrangulateCD = NextUpFunctions.GetSpellCooldown(47476)
	local SoulReaperCD = NextUpFunctions.GetSpellCooldown(130735)
	local MindFreezeCD = NextUpFunctions.GetSpellCooldown(47528)
	local RaiseDeadCD = NextUpFunctions.GetSpellCooldown(46584)
	local OutbreakCD = NextUpFunctions.GetSpellCooldown(77575)

	-- Get buffs
	local _,_,_,BloodCharges = UnitBuff("player", GetSpellInfo(114851))
	if not BloodCharges then BloodCharges = 0 end
	
	local RunicCorruptionUp = UnitBuff("player", GetSpellInfo(51462))
	local KillingMachineUp = UnitBuff("player", GetSpellInfo(51128))
	local FrostPresenceUp = UnitBuff("player", GetSpellInfo(48266))
	local PillarOfFrostUp = UnitBuff("player", GetSpellInfo(51271))
	local RimeUp = UnitBuff("player", GetSpellInfo(59052)) -- The buff is technically called Freezing Fog, but "Rime proc" is the name people use.

	local FrostFeverUp
	local FrostFeverRemaining = 0
	local _,_,_,_,_,_,FrostFeverExpirationTime,FrostFeverCaster = UnitDebuff("target", GetSpellInfo(55095))
	if (FrostFeverCaster == "player") then
		FrostFeverUp = true
		FrostFeverRemaining = (FrostFeverExpirationTime - CurrentTime)
	end
	
	local BloodPlagueUp
	local BloodPlagueRemaining = 0
	local _,_,_,_,_,_,BloodPlagueExpirationTime,BloodPlagueCaster = UnitDebuff("target", GetSpellInfo(55078))
	if (BloodPlagueCaster == "player") then
		BloodPlagueUp = true
		BloodPlagueRemaining = (BloodPlagueExpirationTime - CurrentTime)
	end

	-- Get power levels
	local RunicPower = UnitPower("player", 6)

	local BloodRunes = 0
	local FrostRunes = 0
	local UnholyRunes = 0
	local DeathRunes = 0
	
	local Runes = {}
	
	-- Returns true if the rune in question is ready to be used within one GCD
	local function RuneReady(i)
		if ((CurrentTime - GetRuneCooldown(i)) >= ((select(2,GetRuneCooldown(i))) - GCD)) then
			return true
		else
			return false
		end
	end

	-- Cycles through the 6 runes using the above function to determine how many of each kind are ready to use next GCD.
	for i=1,6,1 do
		Runes[i] = RuneReady(i)
		if (RuneReady(i) and (GetRuneType(i) == 1)) then BloodRunes = BloodRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 2)) then UnholyRunes = UnholyRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 3)) then FrostRunes = FrostRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 4)) then DeathRunes = DeathRunes+1 end
	end

	-- Get Frost Strike cost
	local FrostStrikeCost = 35
	if (GetSpellInfo(GetSpellInfo(50385)) and FrostPresenceUp) then FrostStrikeCost = 20 end -- Improved Frost Presence

	-- Get target's health and spells being cast and check if target is a boss creature
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			if OffhandHasWeapon() then
			
				-- Dual-wielding priority --
			
				-- Blood tap if the player has more than 10 charges and either A: is about to cap Runic Power or B: has a Killing Machine Proc
				if(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and ((RunicPower > 76) or KillingMachineUp) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)

				-- Frost Strike if player has a Killing Machine proc or to avoid getting runic power capped
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (KillingMachineUp or (RunicPower > 88)) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Remove diseases with Plague Leech if one is about to fall off or if Outbreak is just about to come off cooldown so both can be applied right after Leech
				elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3) or (GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= (2*GCD)))) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(123693)
				
				-- Apply diseases with outbreak if one is about to fall off or one is missing
				elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
					_,_,PriorityIcon = GetSpellInfo(77575)
				
				-- Unholy Blight if chosen as tier 1 talent and one disease is about to fall off
				elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
					_,_,PriorityIcon = GetSpellInfo(115989)

				-- Soul Reaper if target hp is below 35 percent
				elseif(GetSpellInfo(GetSpellInfo(130735)) and (SoulReaperCD <= GCD) and (TargetHPPercent <= 35) and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(130735)

				-- Apply Frost fever with Howling Blast
				elseif(GetSpellInfo(GetSpellInfo(49184)) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(49184)
					
				-- Apply Blood Plague with Plague Strike
				elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(45462)

				-- Howling Blast if Rime has procced
				elseif(GetSpellInfo(GetSpellInfo(49184)) and RimeUp) then
					_,_,PriorityIcon = GetSpellInfo(49184)

				-- Frost Strike to avoid getting runic power capped
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower > 76)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Obliterate if player has capped Unholy Runes
				elseif(GetSpellInfo(GetSpellInfo(49020)) and (UnholyRunes == 2) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)
				
				-- Howling Blast if both frost OR both death runes are full (as in the two leftmost runes, not death runes from Plague Leech or Blood Boil. The purpose of this part of the priority is to prevent any of the rune pairs from capping)
				elseif(GetSpellInfo(GetSpellInfo(49184)) and (FrostRunes == 2) or (Runes[1] and Runes[2])) then
					_,_,PriorityIcon = GetSpellInfo(49184)

				-- Horn of Winter
				elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(57330)

				-- Obliterate if player has an Unholy Rune to spend
				elseif(GetSpellInfo(GetSpellInfo(49020)) and (UnholyRunes >= 1) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)

				-- Howling Blast
				elseif(GetSpellInfo(GetSpellInfo(49184)) and FrostRunes >= 1) then
					_,_,PriorityIcon = GetSpellInfo(49184)
				
				-- Frost Strike if player has the Runic Empowerment talent and there is no frost rune or no blood rune
				elseif(GetSpellInfo(GetSpellInfo(49143)) and GetSpellInfo(GetSpellInfo(81229)) and ((FrostRunes == 0) or (not Runes[1] and not Runes[2])) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)
					
				-- Blood tap if the player has 8 or more charges
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges >= 8) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)

				-- Frost Strike if player has the Runic Corruption talent its buff isn't currently up
				elseif(GetSpellInfo(GetSpellInfo(49143)) and GetSpellInfo(GetSpellInfo(51462)) and not RunicCorruptionUp and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)
					
				-- Death And Decay
				elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(43265)

				-- Frost Strike
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower >= (2*FrostStrikeCost))) then
					_,_,PriorityIcon = GetSpellInfo(49143)
					
				end
				
			else
			
				-- Two-handed weapon priority --

				-- Remove diseases with Plague Leech if one is about to fall off
				if(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 1) or (BloodPlagueRemaining < 1)) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(123693)
				
				-- Apply diseases with outbreak if one is missing
				elseif(GetSpellInfo(GetSpellInfo(77575)) and (not BloodPlagueUp or not FrostFeverUp) and (OutbreakCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(77575)
				
				-- Unholy Blight if chosen as tier 1 talent and one disease is missing
				elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and (not FrostFeverUp or not BloodPlagueUp)) then
					_,_,PriorityIcon = GetSpellInfo(115989)

				-- Soul Reaper if target hp is below 35 percent
				elseif(GetSpellInfo(GetSpellInfo(130735)) and (SoulReaperCD <= GCD) and (TargetHPPercent <= 35) and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(130735)

				-- Apply Frost fever with Howling Blast
				elseif(GetSpellInfo(GetSpellInfo(49184)) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(49184)
					
				-- Apply Blood Plague with Plague Strike
				elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(45462)

				-- Howling Blast if Rime has procced
				elseif(GetSpellInfo(GetSpellInfo(49184)) and RimeUp) then
					_,_,PriorityIcon = GetSpellInfo(49184)

				-- Obliterate if Killing Machine has procced
				elseif(GetSpellInfo(GetSpellInfo(49020)) and KillingMachinUp and (UnholyRunes >= 1) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)
				
				-- Blood Tap if Killing Machine has Procced
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges >= 5) and KillingMachineUp and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)
					
				-- Blood Tap if Blood Charges are over 10 and Runic Power is about to cap, IE the player is about to generate more charges
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower > 76) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)

				-- Frost Strike to avoid getting runic power capped
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower > 76)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Obliterate if any rune pair is full
				elseif(GetSpellInfo(GetSpellInfo(49020)) and ((Runes[1] and Runes[2]) or (Runes[3] and Runes[4]) or (Runes[5] and Runes[6])) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)

				-- Remove diseases with Plague Leech if one is about to fall off
				elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3)) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(123693)
				
				-- Apply diseases with outbreak if one is about to fall off
				elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
					_,_,PriorityIcon = GetSpellInfo(77575)
					
				-- Unholy Blight if talented and one disease is about to fall off
				elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
					_,_,PriorityIcon = GetSpellInfo(115989)

				-- Frost Strike if player has the Runic Empowerment talent and there is no frost rune
				elseif(GetSpellInfo(GetSpellInfo(49143)) and GetSpellInfo(GetSpellInfo(81229)) and (FrostRunes == 0) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)
				
				-- Frost Strike if player has the Blood Tap talent and Blood Charges are 10 or fewer
				elseif(GetSpellInfo(GetSpellInfo(49143)) and GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges <= 10) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)
				
				-- Horn of Winter
				elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(57330)

				-- Frost Strike if player has the Runic Corruption talent its buff isn't currently up
				elseif(GetSpellInfo(GetSpellInfo(49143)) and GetSpellInfo(GetSpellInfo(51462)) and not RunicCorruptionUp and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)
				
				-- Obliterate
				elseif(GetSpellInfo(GetSpellInfo(49020)) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)
					
				-- Blood Tap if Blood Charges are above 10 and Runic Power is at the cost of a Frost Strike or higher
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower >= FrostStrikeCost) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)
					
				-- Frost Strike
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Remove diseases with Plague Leech to fill up runes
				elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(123693)

				end
				
			end
			
		else

			-- AoE priority --

				-- Remove diseases with Plague Leech if one is about to fall off
				if(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 1) or (BloodPlagueRemaining < 1)) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(123693)
				
				-- Apply diseases with outbreak if one is missing
				elseif(GetSpellInfo(GetSpellInfo(77575)) and (not BloodPlagueUp or not FrostFeverUp) and (OutbreakCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(77575)
				
				-- Unholy Blight if chosen as tier 1 talent and one disease is missing
				elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and (not FrostFeverUp or not BloodPlagueUp)) then
					_,_,PriorityIcon = GetSpellInfo(115989)

				-- Soul Reaper if target hp is below 35 percent
				elseif(GetSpellInfo(GetSpellInfo(130735)) and (SoulReaperCD <= GCD) and (TargetHPPercent <= 35) and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(130735)

				-- Apply Frost fever with Howling Blast (or Icy Touch if Howling Blast is not learned yet)
				elseif(GetSpellInfo(GetSpellInfo(49184)) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(49184)
					
				-- Apply Blood Plague with Plague Strike
				elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(45462)

				-- Howling Blast if Rime has procced, or if Death / Frost runes are capped
				elseif(GetSpellInfo(GetSpellInfo(49184)) and (RimeUp or (Runes[1] and Runes[2]) or (Runes[5] and Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(49184)

				-- Death And Decay
				elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and (Runes[3] and Runes[4])) then
					_,_,PriorityIcon = GetSpellInfo(43265)

				-- Blood Tap if Blood Charges are over 10 and Runic Power is about to cap, IE the player is about to generate more charges
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower > 76) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)

				-- Frost Strike to avoid getting runic power capped
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower > 76)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Obliterate if Unholy Runes are capped
				elseif(GetSpellInfo(GetSpellInfo(49020)) and (Runes[3] and Runes[4]) and (((UnholyRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((UnholyRunes >= 1) and (FrostRunes >= 1)))) then
					_,_,PriorityIcon = GetSpellInfo(49020)
					
				-- Howling Blast
				elseif(GetSpellInfo(GetSpellInfo(49184)) and FrostRunes >= 1) then
					_,_,PriorityIcon = GetSpellInfo(49184)
				
				-- Death And Decay
				elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) >= 1)) then
					_,_,PriorityIcon = GetSpellInfo(43265)

				-- Blood Tap if Blood Charges are above 10 and Runic Power is at the cost of a Frost Strike or higher
				elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower >= FrostStrikeCost) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
					_,_,PriorityIcon = GetSpellInfo(45529)
					
				-- Frost Strike
				elseif(GetSpellInfo(GetSpellInfo(49143)) and (RunicPower >= FrostStrikeCost)) then
					_,_,PriorityIcon = GetSpellInfo(49143)

				-- Horn of Winter
				elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(57330)

				end

		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Mind Freeze
		if (GetSpellInfo(GetSpellInfo(47528)) and (MindFreezeCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47528)

		-- Strangulate
		elseif (GetSpellInfo(GetSpellInfo(47476)) and (StrangulateCD == 0) and TargetSpellcast and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47476)

		end

	end
	
------------------
-- ReminderIcon --
------------------

	-- Frost Presence
	if (GetSpellInfo(GetSpellInfo(48266)) and not FrostPresenceUp) then
		_,_,ReminderIcon = GetSpellInfo(48266)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Pillar of Frost if target is attackable
		if(GetSpellInfo(GetSpellInfo(51271)) and NextUpFunctions.TargetIsEnemy() and (PillarOfFrostCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(51271)

		-- Raise Dead
		elseif(GetSpellInfo(GetSpellInfo(46584)) and (RaiseDeadCD <= GCD) and PillarOfFrostUp) then
			_,_,ReminderIcon = GetSpellInfo(46584)

		end
		
	end

	return PriorityIcon, InterruptIcon, ReminderIcon, 0
end