function NextUpFunctions.DeathKnightUnholyPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(47541) -- Get Death Coil CD

	-- Get cooldowns
	local DeathAndDecayCD = NextUpFunctions.GetSpellCooldown(43265)
	local UnholyBlightCD = NextUpFunctions.GetSpellCooldown(115989)
	local HornOfWinterCD = NextUpFunctions.GetSpellCooldown(57330)
	local PlagueLeechCD = NextUpFunctions.GetSpellCooldown(123693)
	local StrangulateCD = NextUpFunctions.GetSpellCooldown(47476)
	local SoulReaperCD = NextUpFunctions.GetSpellCooldown(130736)
	local MindFreezeCD = NextUpFunctions.GetSpellCooldown(47528)
	local RaiseDeadCD = NextUpFunctions.GetSpellCooldown(46584)
	local OutbreakCD = NextUpFunctions.GetSpellCooldown(77575)

	-- Get buffs
	local _,_,_,BloodCharges = UnitBuff("player", GetSpellInfo(114851))
	if not BloodCharges then BloodCharges = 0 end
	
	local _,_,_,ShadowInfusionStacks = UnitBuff("player", GetSpellInfo(49572))
	if not ShadowInfusionStacks then ShadowInfusionStacks = 0 end
	
	local SuddenDoomUp = UnitBuff("player", GetSpellInfo(49530))
	local DarkTransformationUp = UnitBuff("pet", GetSpellInfo(63560))
	local UnholyPresenceUp = UnitBuff("player", GetSpellInfo(48265))
	local RunicCorruptionUp = UnitBuff("player", GetSpellInfo(51462))

	local FrostFeverUp
	local FrostFeverRemaining = 0
	local _,_,_,_,_,_,FrostFeverExpirationTime,FrostFeverCaster = UnitDebuff("target", GetSpellInfo(55095))
	if (FrostFeverCaster == "player") then
		FrostFeverUp = true
		FrostFeverRemaining = (FrostFeverExpirationTime - CurrentTime)
	end
	
	local BloodPlagueUp
	local BloodPlagueRemaining = 0
	local _,_,_,_,_,_,BloodPlagueExpirationTime,BloodPlagueCaster = UnitDebuff("target", GetSpellInfo(55078))
	if (BloodPlagueCaster == "player") then
		BloodPlagueUp = true
		BloodPlagueRemaining = (BloodPlagueExpirationTime - CurrentTime)
	end

	-- Get power levels
	local RunicPower = UnitPower("player", 6)

	local BloodRunes = 0
	local FrostRunes = 0
	local UnholyRunes = 0
	local DeathRunes = 0
	
	local Runes = {}
	
	-- Returns true if the rune in question is ready to be used within one GCD
	local function RuneReady(i)
		if ((CurrentTime - GetRuneCooldown(i)) >= ((select(2,GetRuneCooldown(i))) - GCD)) then
			return true
		else
			return false
		end
	end

	-- Cycles through the 6 runes using the above function to determine how many of each kind are ready to use next GCD.
	for i=1,6,1 do
		Runes[i] = RuneReady(i)
		if (RuneReady(i) and (GetRuneType(i) == 1)) then BloodRunes = BloodRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 2)) then UnholyRunes = UnholyRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 3)) then FrostRunes = FrostRunes+1 end
		if (RuneReady(i) and (GetRuneType(i) == 4)) then DeathRunes = DeathRunes+1 end
	end

	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --
			
			-- Blood Tap if the player has more than 10 charges and has 32 or more Runic Power
			if(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower >= 32) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Remove diseases with Plague Leech if one is about to fall off or if Outbreak is just about to come off cooldown so both can be applied right after Leech
			elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3) or (GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= (2*GCD)))) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(123693)
			
			-- Apply diseases with outbreak if one is about to fall off or one is missing
			elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(77575)
			
			-- Unholy Blight if chosen as tier 1 talent and one disease is about to fall off
			elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(115989)

			-- Soul Reaper if target hp is below 35 percent
			elseif(GetSpellInfo(GetSpellInfo(130736)) and (SoulReaperCD <= GCD) and (TargetHPPercent <= 35) and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(130736)

			-- Apply Blood Plague and Frost Fever with Plague Strike
			elseif(GetSpellInfo(GetSpellInfo(45462)) and (not BloodPlagueUp or not FrostFeverUp) and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45462)

			-- Dark Transformation if 5 Shadow Infusion Stacks
			elseif(GetSpellInfo(GetSpellInfo(63560)) and (ShadowInfusionStacks == 5) and ((UnholyRunes + DeathRunes) > 0)) then
				_,_,PriorityIcon = GetSpellInfo(63560)

			-- Blood Tap if the player has 5 Shadow Infusion Stacks
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (ShadowInfusionStacks == 5) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Death Coil to avoid getting runic power capped
			elseif(GetSpellInfo(GetSpellInfo(47541)) and (RunicPower > 90)) then
				_,_,PriorityIcon = GetSpellInfo(47541)

			-- Death And Decay if both Unholy Runes are full
			elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and (Runes[3] and Runes[4])) then
				_,_,PriorityIcon = GetSpellInfo(43265)

			-- Scourge Strike if both Unholy Runes are full
			elseif(GetSpellInfo(GetSpellInfo(55090)) and (Runes[3] and Runes[4])) then
				_,_,PriorityIcon = GetSpellInfo(55090)

			-- Festering Strike if both Blood Runes and both Frost Runes are full
			elseif(GetSpellInfo(GetSpellInfo(85948)) and (Runes[1] and Runes[2]) and (Runes[5] and Runes[6])) then
				_,_,PriorityIcon = GetSpellInfo(85948)

			-- Death And Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) > 0)) then
				_,_,PriorityIcon = GetSpellInfo(43265)

			-- Blood Tap if Death And Decay is off cooldown
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (DeathAndDecayCD == 0) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Death Coil if Sudden Doom has procced or Dark Transformation is down and player has at most one Unholy Rune
			elseif(GetSpellInfo(GetSpellInfo(47541)) and (SuddenDoomUp or (not DarkTransformationUp and (not Runes[3] or not Runes[4]))) and (RunicPower >= 32)) then
				_,_,PriorityIcon = GetSpellInfo(47541)

			-- Scourge Strike
			elseif(GetSpellInfo(GetSpellInfo(55090)) and ((UnholyRunes + DeathRunes) > 0 )) then
				_,_,PriorityIcon = GetSpellInfo(55090)

			-- Festering Strike
			elseif(GetSpellInfo(GetSpellInfo(85948)) and (((BloodRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((BloodRunes >= 1) and (FrostRunes >= 1)))) then
				_,_,PriorityIcon = GetSpellInfo(85948)

			-- Horn of Winter
			elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(57330)

			-- Death Coil if Dark Transformation is down
			elseif(GetSpellInfo(GetSpellInfo(47541)) and not DarkTransformationUp and (RunicPower >= 32)) then
				_,_,PriorityIcon = GetSpellInfo(47541)

			-- Blood tap if the player has 8 or more charges
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges >= 8) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			end
				
		else

			-- AoE priority --

			-- Blood Tap if the player has more than 10 charges and has 32 or more Runic Power
			if(GetSpellInfo(GetSpellInfo(45529)) and (BloodCharges > 10) and (RunicPower >= 32) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Remove diseases with Plague Leech if one is about to fall off or if Outbreak is just about to come off cooldown so both can be applied right after Leech
			elseif(GetSpellInfo(GetSpellInfo(123693)) and (PlagueLeechCD <= GCD) and FrostFeverUp and BloodPlagueUp and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3) or (GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= (2*GCD)))) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(123693)
			
			-- Apply diseases with outbreak if one is about to fall off or one is missing
			elseif(GetSpellInfo(GetSpellInfo(77575)) and (OutbreakCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(77575)
			
			-- Unholy Blight if chosen as tier 1 talent and one disease is about to fall off
			elseif(GetSpellInfo(GetSpellInfo(115989)) and (UnholyBlightCD <= GCD) and ((FrostFeverRemaining < 3) or (BloodPlagueRemaining < 3))) then
				_,_,PriorityIcon = GetSpellInfo(115989)

			-- Apply Frost fever with Icy Touch
			elseif(GetSpellInfo(GetSpellInfo(45477)) and not FrostFeverUp and ((FrostRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45477)
				
			-- Apply Blood Plague with Plague Strike
			elseif(GetSpellInfo(GetSpellInfo(45462)) and not BloodPlagueUp and ((UnholyRunes + DeathRunes) >= 1)) then
				_,_,PriorityIcon = GetSpellInfo(45462)

			-- Dark Transformation if 5 Shadow Infusion Stacks
			elseif(GetSpellInfo(GetSpellInfo(63560)) and (ShadowInfusionStacks == 5) and ((UnholyRunes + DeathRunes) > 0)) then
				_,_,PriorityIcon = GetSpellInfo(63560)

			-- Blood Tap if the player has 5 Shadow Infusion Stacks
			elseif(GetSpellInfo(GetSpellInfo(45529)) and (ShadowInfusionStacks == 5) and ((not Runes[1] and not Runes[2]) or (not Runes[3] and not Runes[4]) or (not Runes[5] and not Runes[6]))) then
				_,_,PriorityIcon = GetSpellInfo(45529)

			-- Death And Decay
			elseif(GetSpellInfo(GetSpellInfo(43265)) and (DeathAndDecayCD <= GCD) and ((UnholyRunes + DeathRunes) > 0)) then
				_,_,PriorityIcon = GetSpellInfo(43265)

			-- Festering Strike to convert useless Frost Runes to Death Runes
			elseif(GetSpellInfo(GetSpellInfo(85948)) and (FrostRunes > 0) and (((BloodRunes + DeathRunes) >= 2) or ((FrostRunes + DeathRunes) >= 2) or ((BloodRunes >= 1) and (FrostRunes >= 1)))) then
				_,_,PriorityIcon = GetSpellInfo(85948)

			-- Blood Boil
			elseif(GetSpellInfo(GetSpellInfo(48721)) and ((BloodRunes + DeathRunes) > 0)) then
				_,_,PriorityIcon = GetSpellInfo(48721)

			-- Scourge Strike if both Unholy Runes are full
			elseif(GetSpellInfo(GetSpellInfo(55090)) and (Runes[3] and Runes[4])) then
				_,_,PriorityIcon = GetSpellInfo(55090)

			-- Death Coil to avoid getting runic power capped
			elseif(GetSpellInfo(GetSpellInfo(47541)) and (RunicPower > 90)) then
				_,_,PriorityIcon = GetSpellInfo(47541)

			-- Death Coil if Dark Transformation is down
			elseif(GetSpellInfo(GetSpellInfo(47541)) and not DarkTransformationUp and (RunicPower >= 32)) then
				_,_,PriorityIcon = GetSpellInfo(47541)

			-- Horn of Winter
			elseif(GetSpellInfo(GetSpellInfo(57330)) and (HornOfWinterCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(57330)

			end

		end

-------------------
-- InterruptIcon --
-------------------

		-- Mind Freeze
		if (GetSpellInfo(GetSpellInfo(47528)) and (MindFreezeCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47528)

		-- Strangulate
		elseif (GetSpellInfo(GetSpellInfo(47476)) and (StrangulateCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(47476)

		end

	end
	
------------------
-- ReminderIcon --
------------------

	-- Unholy Presence
	if (GetSpellInfo(GetSpellInfo(48265)) and not UnholyPresenceUp) then
		_,_,ReminderIcon = GetSpellInfo(48265)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then
		
		-- Raise Dead
		if(GetSpellInfo(GetSpellInfo(46584)) and not UnitExists("pet") and (RaiseDeadCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(46584)

		end
		
	end

	return PriorityIcon, InterruptIcon, ReminderIcon, ShadowInfusionStacks
end