function NextUpFunctions.DruidBalancePriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(5176) -- Get Wrath CD

	-- Get cooldowns
	local WildMushroomDetonateCD = NextUpFunctions.GetSpellCooldown(88751)
	local CelestialAlignmentCD = NextUpFunctions.GetSpellCooldown(112071)
	local NaturesSwiftnessCD = NextUpFunctions.GetSpellCooldown(132158)
	local ForceOfNatureCD = NextUpFunctions.GetSpellCooldown(106737)
	local NaturesVigilCD = NextUpFunctions.GetSpellCooldown(124974)
	local IncarnationCD = NextUpFunctions.GetSpellCooldown(106731)
	local StarsurgeCD = NextUpFunctions.GetSpellCooldown(78674)
	local SolarBeamCD = NextUpFunctions.GetSpellCooldown(78675)
	local StarfallCD = NextUpFunctions.GetSpellCooldown(48505)

	-- Get buffs and debuffs
	local ChosenOfEluneUp = UnitBuff("player", GetSpellInfo(102560))
	local ShootingStarsUp = UnitBuff("player", GetSpellInfo(93399))
	local SolarEclipseUp = UnitBuff("player", GetSpellInfo(48517))
	local LunarEclipseUp = UnitBuff("player", GetSpellInfo(48518))
	local StarfallUp = UnitBuff("player", GetSpellInfo(48505))

	local _,_,_,DreamOfCenariusCharges = UnitBuff("player", GetSpellInfo(108373))
	if not DreamOfCenariusCharges then DreamOfCenariusCharges = 0 end

	local CelestialAlignmentUp,_,_,_,_,_,CelestialAlignmentExpirationTime = UnitBuff("player", GetSpellInfo(106952))
	local CelestialAlignmentRemaining = 0
	if CelestialAlignmentUp then CelestialAlignmentRemaining = (CelestialAlignmentExpirationTime - CurrentTime) end
	
	local MoonfireUp
	local MoonfireRemaining = 0
	local _,_,_,_,_,_,MoonfireExpirationTime,MoonfireCaster = UnitDebuff("target", GetSpellInfo(8921))
	if (MoonfireCaster == "player") then
		MoonfireUp = true
		MoonfireRemaining = (MoonfireExpirationTime - CurrentTime)
	end

	local SunfireUp
	local SunfireRemaining = 0
	local _,_,_,_,_,_,SunfireExpirationTime,SunfireCaster = UnitDebuff("target", GetSpellInfo(93402))
	if (SunfireCaster == "player") then
		SunfireUp = true
		SunfireRemaining = (SunfireExpirationTime - CurrentTime)
	end

	-- Get Mushroom info
	local WildMushroomPlaced = GetTotemInfo(1)

	-- Get Balance
	local Balance = UnitPower("player", 8)

		-- Adjust ahead of time if a Solar or Lunar Eclipse will be reached by the spell currently being cast or if an eclipse state will be lost by the cast
		-- (double solar / lunar power gained outside eclipses if player has Euphoria)

		-- Starfire
		if((UnitCastingInfo("player") == GetSpellInfo(2912)) and ((Balance >= 80) or ((Balance >= 60) and GetSpellInfo(GetSpellInfo(81062))))) then
			SolarEclipseUp = true
		end
		if((UnitCastingInfo("player") == GetSpellInfo(2912)) and (Balance >= -20)) then
			LunarEclipseUp = false
		end
		
		-- Wrath
		if((UnitCastingInfo("player") == GetSpellInfo(5176)) and ((Balance <= -85) or ((Balance <= -70) and GetSpellInfo(GetSpellInfo(81062))))) then
			LunarEclipseUp = true
			StarfallCD = 0
		end
		if((UnitCastingInfo("player") == GetSpellInfo(5176)) and (Balance <= 15)) then
			SolarEclipseUp = false
		end

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Force of Nature if specced
			if(GetSpellInfo(GetSpellInfo(106737)) and (ForceOfNatureCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(106737)

			-- Starfall if not using already
			elseif(GetSpellInfo(GetSpellInfo(48505)) and not StarfallUp and (StarfallCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(48505)

			-- Detonate Mushrooms if Solar Eclipse is up and there's at least one mushroom to detonate
			elseif(GetSpellInfo(GetSpellInfo(88751)) and SolarEclipseUp and WildMushroomPlaced and (WildMushroomDetonateCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(88751)

			-- Healing Touch if specced into Dream of Cenarius but the buff isn't up, and mana is above 25%
			elseif(GetSpellInfo(GetSpellInfo(5185)) and GetSpellInfo(GetSpellInfo(108373)) and (DreamOfCenariusCharges == 0) and (NextUpFunctions.GetPlayerManaPercent() > 25)) then
				_,_,PriorityIcon = GetSpellInfo(5185)

			-- Incarnation if either Eclipse is up
			elseif(GetSpellInfo(GetSpellInfo(106731)) and (SolarEclipseUp or LunarEclipseUp) and (IncarnationCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(106731)

			-- Celestial Alignment if neither eclipse is up. Used in conjunction with Incarnation if specced and player doesn't have to wait longer than 10 seconds for it.
			elseif(GetSpellInfo(GetSpellInfo(112071)) and not (SolarEclipseUp or LunarEclipseUp) and (ChosenOfEluneUp or not GetSpellInfo(GetSpellInfo(106731)) or (IncarnationCD > 10)) and (CelestialAlignmentCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(112071)

			-- Starsurge if instant thanks to Shooting Stars
			elseif(GetSpellInfo(GetSpellInfo(78674)) and ShootingStarsUp) then
				_,_,PriorityIcon = GetSpellInfo(78674)

			-- Moonfire if it's about to expire and Lunar Eclipse or Celestial Alignment is up
			elseif(GetSpellInfo(GetSpellInfo(8921)) and (MoonfireRemaining < 3) and (LunarEclipseUp or CelestialAlignmentUp)) then
				_,_,PriorityIcon = GetSpellInfo(8921)

			-- Sunfire if it's about to expire and Solar Eclipse is up
			elseif(GetSpellInfo(GetSpellInfo(93402)) and (SunfireRemaining < 3) and SolarEclipseUp) then
				_,_,PriorityIcon = GetSpellInfo(93402)

			-- Starsurge
			elseif(GetSpellInfo(GetSpellInfo(78674)) and (UnitCastingInfo("player") ~= GetSpellInfo(78674)) and (StarsurgeCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(78674)

			-- Starfire if Celestial Alignment is up and Starfire's cast time is shorter than Celestial Alignment's duration
			elseif(GetSpellInfo(GetSpellInfo(2912)) and CelestialAlignmentUp and ((select(7, GetSpellInfo(2912))) / 1000 < CelestialAlignmentRemaining)) then
				_,_,PriorityIcon = GetSpellInfo(2912)
				
			-- Wrath if Celestial Alignment is up and Wrath's cast time is shorter than Celestial Alignment's duration
			elseif(GetSpellInfo(GetSpellInfo(5176)) and CelestialAlignmentUp and ((select(7, GetSpellInfo(5176))) / 1000 < CelestialAlignmentRemaining)) then
				_,_,PriorityIcon = GetSpellInfo(5176)

			-- Starfire if moving towards Solar Eclipse or there's no direction and the balance meter is closer to Solar Eclipse
			elseif(GetSpellInfo(GetSpellInfo(2912)) and ((GetEclipseDirection() == "sun") or ((GetEclipseDirection() == "none") and (Balance > 0)))) then
				_,_,PriorityIcon = GetSpellInfo(2912)

			-- Wrath if moving towards Lunar Eclipse or there's no direction and the balance meter is closer to Lunar Eclipse
			elseif(GetSpellInfo(GetSpellInfo(5176)) and ((GetEclipseDirection() == "moon") or ((GetEclipseDirection() == "none") and (Balance <= 0)))) then
				_,_,PriorityIcon = GetSpellInfo(5176)

			end

		else

			-- AoE priority --

			-- Starfall if not using already
			if(GetSpellInfo(GetSpellInfo(48505)) and not StarfallUp and (StarfallCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(48505)

			-- Detonate Mushrooms if Solar Eclipse is up and there's at least one mushroom to detonate
			elseif(GetSpellInfo(GetSpellInfo(88751)) and SolarEclipseUp and WildMushroomPlaced and (WildMushroomDetonateCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(88751)

			-- Healing Touch if specced into Dream of Cenarius but the buff isn't up, and mana is above 25%
			elseif(GetSpellInfo(GetSpellInfo(5185)) and GetSpellInfo(GetSpellInfo(108373)) and (DreamOfCenariusCharges == 0) and (NextUpFunctions.GetPlayerManaPercent() > 25)) then
				_,_,PriorityIcon = GetSpellInfo(5185)

			-- Incarnation if either Eclipse is up
			elseif(GetSpellInfo(GetSpellInfo(106731)) and (SolarEclipseUp or LunarEclipseUp) and (IncarnationCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(106731)

			-- Celestial Alignment if neither eclipse is up. Used in conjunction with Incarnation if specced and player doesn't have to wait longer than 10 seconds for it.
			elseif(GetSpellInfo(GetSpellInfo(112071)) and not (SolarEclipseUp or LunarEclipseUp) and (ChosenOfEluneUp or not GetSpellInfo(GetSpellInfo(106731)) or (IncarnationCD > 10)) and (CelestialAlignmentCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(112071)

			-- Starsurge if instant thanks to Shooting Stars
			elseif(GetSpellInfo(GetSpellInfo(78674)) and ShootingStarsUp) then
				_,_,PriorityIcon = GetSpellInfo(78674)

			-- Moonfire if it's about to expire and Lunar Eclipse or Celestial Alignment is up
			elseif(GetSpellInfo(GetSpellInfo(8921)) and (MoonfireRemaining < 3) and (LunarEclipseUp or CelestialAlignmentUp)) then
				_,_,PriorityIcon = GetSpellInfo(8921)

			-- Sunfire if it's about to expire and Solar Eclipse is up
			elseif(GetSpellInfo(GetSpellInfo(93402)) and (SunfireRemaining < 3) and SolarEclipseUp) then
				_,_,PriorityIcon = GetSpellInfo(93402)
			
			-- Moonfire if it's about to expire
			elseif(GetSpellInfo(GetSpellInfo(8921)) and (MoonfireRemaining < 3)) then
				_,_,PriorityIcon = GetSpellInfo(8921)

			-- Sunfire if it's about to expire
			elseif(GetSpellInfo(GetSpellInfo(93402)) and (SunfireRemaining < 3)) then
				_,_,PriorityIcon = GetSpellInfo(93402)
			
			-- Hurricane
			elseif(GetSpellInfo(GetSpellInfo(16914)) and (NextUpFunctions.GetPlayerManaPercent() > 15)) then
				_,_,PriorityIcon = GetSpellInfo(16914)

			-- Starfire if moving towards Solar Eclipse or there's no direction and the balance meter is closer to Solar Eclipse
			elseif(GetSpellInfo(GetSpellInfo(2912)) and ((GetEclipseDirection() == "sun") or ((GetEclipseDirection() == "none") and (Balance > 0)))) then
				_,_,PriorityIcon = GetSpellInfo(2912)

			-- Wrath if moving towards Lunar Eclipse or there's no direction and the balance meter is closer to Lunar Eclipse
			elseif(GetSpellInfo(GetSpellInfo(5176)) and ((GetEclipseDirection() == "moon") or ((GetEclipseDirection() == "none") and (Balance <= 0)))) then
				_,_,PriorityIcon = GetSpellInfo(5176)

			end

		end	
			
-------------------
-- InterruptIcon --
-------------------

		-- Solar Beam if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(78675)) and (SolarBeamCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(78675)

		-- Nature's Swiftness if specced into it and Dream of Cenarius
		elseif(GetSpellInfo(GetSpellInfo(132158)) and GetSpellInfo(GetSpellInfo(108373)) and (NaturesSwiftnessCD <= GCD)) then
			_,_,InterruptIcon = GetSpellInfo(132158)

		end
		
	end
	
------------------
-- ReminderIcon --
------------------

	-- Mark of the Wild
	if(GetSpellInfo(GetSpellInfo(1126)) and not NextUpFunctions.StatBuffUp()) then
		_,_,ReminderIcon = GetSpellInfo(1126)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Nature's Vigil if specced
		if(GetSpellInfo(GetSpellInfo(124974)) and (NaturesVigilCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(124974)

		end

	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon

end