function NextUpFunctions.DruidGuardianPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(5176) -- Get Wrath CD
	
	-- Get cooldowns
	local SavageDefenseCD = NextUpFunctions.GetSpellCooldown(62606)
	local FaerieSwarmCD = NextUpFunctions.GetSpellCooldown(102355)
	local SkullBashCD = NextUpFunctions.GetSpellCooldown(106839)
	local FaerieFireCD = NextUpFunctions.GetSpellCooldown(770)
	local LacerateCD = NextUpFunctions.GetSpellCooldown(33745)
	local MangleCD = NextUpFunctions.GetSpellCooldown(33878)
	local ThrashCD = NextUpFunctions.GetSpellCooldown(77758)
	local EnrageCD = NextUpFunctions.GetSpellCooldown(5229)
	local SwipeCD = NextUpFunctions.GetSpellCooldown(779)
	local MaulCD = NextUpFunctions.GetSpellCooldown(6807)

	-- Get buffs and target debuffs
	local SavageDefenseUp = UnitBuff("player", GetSpellInfo(62606))
	local WeakenedBlowsUp,_,_,_,_,_,WeakenedBlowsExpirationTime = UnitDebuff("target", GetSpellInfo(115798))
	local WeakenedBlowsRemaining = 0
	if WeakenedBlowsUp then WeakenedBlowsRemaining = (WeakenedBlowsExpirationTime - CurrentTime) end
	
	-- Get power levels
	local Rage = UnitPower("player", 1)

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Mangle
			if(GetSpellInfo(GetSpellInfo(33878)) and MangleCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(33878)

			-- Lacerate
			elseif(GetSpellInfo(GetSpellInfo(33745)) and LacerateCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(33745)

			-- Thrash if there's no Weakened Blows debuff on target
			elseif(GetSpellInfo(GetSpellInfo(77758)) and ThrashCD <= GCD and not WeakenedBlowsUp) then
				_,_,PriorityIcon = GetSpellInfo(77758)
			
			-- Faerie Swarm if Thrash has at least 6 seconds remaining and the talent is picked
			elseif(GetSpellInfo(GetSpellInfo(102355)) and (WeakenedBlowsRemaining >= 6) and (FaerieSwarmCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(102355)

			-- Faerie Fire if not talented
			elseif(GetSpellInfo(GetSpellInfo(770)) and (WeakenedBlowsRemaining >= 6) and (FaerieFireCD <= GCD)) then
					_,_,PriorityIcon = GetSpellInfo(770)

			-- Thrash as a filler
			elseif(GetSpellInfo(GetSpellInfo(77758)) and ThrashCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(77758)

			end

		else

			-- AoE priority --

			-- Mangle
			if(GetSpellInfo(GetSpellInfo(33878)) and MangleCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(33878)

			-- Thrash
			elseif(GetSpellInfo(GetSpellInfo(77758)) and ThrashCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(77758)

			-- Swipe
			elseif(GetSpellInfo(GetSpellInfo(779)) and SwipeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(779)

			-- Lacerate
			elseif(GetSpellInfo(GetSpellInfo(33745)) and LacerateCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(33745)

			end
		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Skull Bash if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(106839)) and UnitIsEnemy("target", "player") and (SkullBashCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(106839)

		-- Keep Savage Defense up
		elseif (GetSpellInfo(GetSpellInfo(62606)) and not SavageDefenseUp and (SavageDefenseCD == 0) and (Rage > 60)) then
			_,_,InterruptIcon = GetSpellInfo(62606)

		-- Maul if there's still excess rage even with Savage Defense up
		elseif (GetSpellInfo(GetSpellInfo(6807)) and UnitIsEnemy("target", "player") and (MaulCD == 0) and (Rage > 70)) then
			_,_,InterruptIcon = GetSpellInfo(6807)

		end
	end
	
------------------
-- ReminderIcon --
------------------

	-- Mark of the Wild
	if(GetSpellInfo(GetSpellInfo(1126)) and not NextUpFunctions.StatBuffUp()) then
		_,_,ReminderIcon = GetSpellInfo(1126)

	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon, 0, nil
end