function NextUpFunctions.PaladinRetributionPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(105361) -- Get Seal of Command CD
	
	-- Get cooldowns
	local GuardianOfAncientKingsCD = NextUpFunctions.GetSpellCooldown(86698)
	local ExecutionSentenceCD = NextUpFunctions.GetSpellCooldown(114157)
	local CrusaderStrikeCD = NextUpFunctions.GetSpellCooldown(35395)
	local HammerOfWrathCD = NextUpFunctions.GetSpellCooldown(24275)
	local AvengingWrathCD = NextUpFunctions.GetSpellCooldown(31884)
	local JudgmentCD = NextUpFunctions.GetSpellCooldown(20271)
	local ExorcismCD = NextUpFunctions.GetSpellCooldown(879)
	local RebukeCD = NextUpFunctions.GetSpellCooldown(96231)

	-- Get buffs
	local _,_,_,_,_,_,_,BlessingOfKingsSource = UnitBuff("player", GetSpellInfo(20217))
	local _,_,_,_,_,_,_,BlessingOfMightSource = UnitBuff("player", GetSpellInfo(19740))

	local InquisitionUp,_,_,_,_,_,InquisitionExpirationTime = UnitBuff("player", GetSpellInfo(84963))
	local InquisitionRemaining = 0
	if InquisitionUp then InquisitionRemaining = (InquisitionExpirationTime - CurrentTime) end

	local RighteousFuryUp = UnitBuff("player", GetSpellInfo(25780))
	local DivinePurposeUp = UnitBuff("player", GetSpellInfo(86172))
	local AvengingWrathUp = UnitBuff("player", GetSpellInfo(31884))
	local TheArtOfWarUp = UnitBuff("player", GetSpellInfo(87138))

	-- Get power levels
	local HolyPower = UnitPower("player", 9)
	local ThreeHolyPower = false
	if ((HolyPower >= 3) or (DivinePurposeUp)) then ThreeHolyPower = true end

	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Inquisition if it's not up or at three holy power if it's about to expire
			if(GetSpellInfo(GetSpellInfo(84963)) and (not InquisitionUp or ((InquisitionRemaining <= 2) and ThreeHolyPower))) then
				_,_,PriorityIcon = GetSpellInfo(84963)

			-- Execution Sentence if talent is active and Inquisition is up
			elseif(GetSpellInfo(GetSpellInfo(114157)) and (ExecutionSentenceCD <= GCD) and InquisitionUp) then
				_,_,PriorityIcon = GetSpellInfo(114157)

			-- Templar's Verdict if 5 holy power
			elseif(GetSpellInfo(GetSpellInfo(85256)) and (HolyPower == 5)) then
				_,_,PriorityIcon = GetSpellInfo(85256)

			-- Hammer of Wrath
			elseif(GetSpellInfo(GetSpellInfo(24275)) and HammerOfWrathCD <= GCD and (AvengingWrathUp or TargetHPPercent <= 20)) then
				_,_,PriorityIcon = GetSpellInfo(24275)

			-- Exorcism
			elseif(GetSpellInfo(GetSpellInfo(879)) and ((ExorcismCD <= GCD) or TheArtOfWarUp)) then
				_,_,PriorityIcon = GetSpellInfo(879)

			-- Crusader Strike
			elseif(GetSpellInfo(GetSpellInfo(35395)) and CrusaderStrikeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(35395)

			-- Judgment
			elseif(GetSpellInfo(GetSpellInfo(20271)) and JudgmentCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(20271)

			-- Templar's Verdict if Inquisition buff has more then 4 seconds left on it
			elseif(GetSpellInfo(85256) and ThreeHolyPower and (InquisitionRemaining > 4)) then
				_,_,PriorityIcon = GetSpellInfo(85256)

			end

		else

			-- AoE priority --

			-- Inquisition
			if(GetSpellInfo(GetSpellInfo(84963)) and not InquisitionUp and ThreeHolyPower) then
				_,_,PriorityIcon = GetSpellInfo(84963)

			-- Divine Storm if 5 holy power
			elseif(GetSpellInfo(GetSpellInfo(53385)) and (HolyPower == 5)) then
				_,_,PriorityIcon = GetSpellInfo(53385)

			-- Hammer of Wrath
			elseif(GetSpellInfo(GetSpellInfo(24275)) and HammerOfWrathCD <= GCD and (AvengingWrathUp or TargetHPPercent <= 20)) then
				_,_,PriorityIcon = GetSpellInfo(24275)

			-- Exorcism
			elseif(GetSpellInfo(GetSpellInfo(879)) and ((ExorcismCD <= GCD) or TheArtOfWarUp)) then
				_,_,PriorityIcon = GetSpellInfo(879)

			-- Hammer of the Righteous
			elseif(GetSpellInfo(GetSpellInfo(53595)) and CrusaderStrikeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(53595)

			-- Judgment
			elseif(GetSpellInfo(GetSpellInfo(20271)) and JudgmentCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(20271)

			-- Divine Storm
			elseif(GetSpellInfo(GetSpellInfo(53385)) and ThreeHolyPower) then
				_,_,PriorityIcon = GetSpellInfo(53385)

			end
		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Rebuke
		if (GetSpellInfo(GetSpellInfo(96231)) and (RebukeCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(96231)

		end
	
	end
		
------------------
-- ReminderIcon --
------------------

 	-- Make sure Righteous Fury is off
	if(RighteousFuryUp) then
		_,_,ReminderIcon = GetSpellInfo(25780)

	-- Blessing of Kings
	elseif(GetSpellInfo(GetSpellInfo(20217)) and not NextUpFunctions.StatBuffUp() and not (BlessingOfMightSource == "player")) then
		_,_,ReminderIcon = GetSpellInfo(20217)

	-- Blessing of Might
	elseif(GetSpellInfo(GetSpellInfo(19740)) and not NextUpFunctions.MasteryBuffUp() and not (BlessingOfKingsSource == "player")) then
		_,_,ReminderIcon = GetSpellInfo(19740)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Avengning Wrath if Inquisition is up
		if(GetSpellInfo(GetSpellInfo(31884)) and InquisitionUp and (AvengingWrathCD == 0)) then
			_,_,ReminderIcon = GetSpellInfo(31884)

		-- Guardian of Ancient Kings if Avengning Wrath is up
		elseif(GetSpellInfo(GetSpellInfo(86698)) and AvengingWrathUp and (GuardianOfAncientKingsCD == 0)) then
			_,_,ReminderIcon = GetSpellInfo(86698)

		end

	end

	return PriorityIcon, InterruptIcon, ReminderIcon, HolyPower
end