function NextUpFunctions.RogueAssassinationPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local ResourceLow
	local CurrentTime = GetTime()
	local GCD = 1
	
	-- Get cooldowns
	local MarkedForDeathCD = NextUpFunctions.GetSpellCooldown(137619)
	local ShadowBladesCD = NextUpFunctions.GetSpellCooldown(121471)
	local PreparationCD = NextUpFunctions.GetSpellCooldown(14185)
	local VendettaCD = NextUpFunctions.GetSpellCooldown(79140)
	local VanishCD = NextUpFunctions.GetSpellCooldown(1856)
	local KickCD = NextUpFunctions.GetSpellCooldown(1766)

	-- Get buffs
	local ShadowBladesUp = UnitBuff("player", GetSpellInfo(121471))
	local DeadlyPoisonUp = UnitBuff("player", GetSpellInfo(2823))
	local BlindsideUp = UnitBuff("player", GetSpellInfo(121152))
	local StealthUp = UnitBuff("player", GetSpellInfo(1784))
	local VanishUp = UnitBuff("player", GetSpellInfo(1856))

	local _,_,_,AnticipationCharges = UnitBuff("player", GetSpellInfo(114015))
	if not AnticipationCharges then AnticipationCharges = 0 end
	
	local SliceAndDiceUp,_,_,_,_,_,SliceAndDiceExpirationTime = UnitBuff("player", GetSpellInfo(5171))
	local SliceAndDiceRemaining = 0
	if SliceAndDiceUp then SliceAndDiceRemaining = (SliceAndDiceExpirationTime - CurrentTime) end
	
	local RuptureUp
	local RuptureRemaining = 0
	local _,_,_,_,_,_,RuptureExpirationTime,RuptureCaster = UnitDebuff("target", GetSpellInfo(1943))
	if (RuptureCaster == "player") then
		RuptureUp = true
		RuptureRemaining = (RuptureExpirationTime - CurrentTime)
	end
	
	local MainHandPoisonUp,_,_,OffHandPoisonUp = GetWeaponEnchantInfo()
	
	local WeakenedArmorUp 
	if (UnitDebuff("target", GetSpellInfo(113746))) then
		WeakenedArmorUp = true
	end
	
	-- Get power levels
	local Energy = UnitPower("player", 3)
	local ComboPoints = GetComboPoints("player", "target")
	
	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Ambush if stealthed
			if(GetSpellInfo(GetSpellInfo(8676)) and (StealthUp or VanishUp)) then
				_,_,PriorityIcon = GetSpellInfo(8676)
				if Energy < 60 then ResourceLow = true end

			-- Slice and Dice if it's just about to wear off or isn't up, no matter how many combo points
			elseif(GetSpellInfo(GetSpellInfo(5171)) and (ComboPoints > 0) and (SliceAndDiceRemaining < 2)) then
				_,_,PriorityIcon = GetSpellInfo(5171)
				if Energy < 25 then ResourceLow = true end

			-- Dispatch to avoid energy capping if target health is below 35% or Blindside procced
			elseif(GetSpellInfo(GetSpellInfo(111240)) and (RuptureRemaining > 3) and (Energy > 90) and (BlindsideUp or (TargetHPPercent <= 35))) then
				_,_,PriorityIcon = GetSpellInfo(111240)

			-- Mutilate to avoid energy capping
			elseif(GetSpellInfo(GetSpellInfo(1329)) and (RuptureRemaining > 3) and (Energy > 90)) then
				_,_,PriorityIcon = GetSpellInfo(1329)

			-- Marked for death if no Combo Points
			elseif(GetSpellInfo(GetSpellInfo(137619)) and (MarkedForDeathCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(137619)

			-- Rupture if it's just about to wear off or isn't up no matter how many combo points or if 5 combo points and it'll wear off soon-ish
			elseif(GetSpellInfo(GetSpellInfo(1943)) and (((ComboPoints > 0) and (RuptureRemaining < 2)) or ((ComboPoints == 5) and (RuptureRemaining < 5)))) then
				_,_,PriorityIcon = GetSpellInfo(1943)
				if Energy < 25 then ResourceLow = true end

			else

				-- Vendetta
				if(GetSpellInfo(GetSpellInfo(79140)) and (VendettaCD <= GCD)) then
					_,_,ReminderIcon = GetSpellInfo(79140)

				end
				
				-- Envenom with 5 or more Combo points, or with 2 or more if Slice and Dice needs refreshing soon.
				if(GetSpellInfo(GetSpellInfo(32645)) and ((ComboPoints >= 5) or ((ComboPoints >= 2) and (SliceAndDiceRemaining < 3)))) then
					_,_,PriorityIcon = GetSpellInfo(32645)
					if Energy < 35 then ResourceLow = true end

				-- Dispatch if target health is below 35% or Blindside procced, and Combo Points is below 5
				elseif(GetSpellInfo(GetSpellInfo(111240)) and (ComboPoints < 5) and (BlindsideUp or ((TargetHPPercent <= 35)))) then
					_,_,PriorityIcon = GetSpellInfo(111240)
					if ((Energy < 30) and not BlindsideUp) then ResourceLow = true end

				-- Keep weakaned armor debuff up on target if noone else does
				elseif(GetSpellInfo(GetSpellInfo(8647)) and not WeakenedArmorUp) then
					_,_,PriorityIcon = GetSpellInfo(8647)
					if Energy < 20 then ResourceLow = true end

				-- Build Combo points with Mutilate
				elseif(GetSpellInfo(GetSpellInfo(1329))) then
					_,_,PriorityIcon = GetSpellInfo(1329)
					if Energy < 55 then ResourceLow = true end

				end
					
			end

		else

			-- AoE priority --

			-- Crimson Tempest at 5 combo points
			if(GetSpellInfo(GetSpellInfo(121411)) and (ComboPoints == 5)) then
				_,_,PriorityIcon = GetSpellInfo(121411)
				if Energy < 35 then ResourceLow = true end

			-- Build combo points with Fan of Knives
			elseif(GetSpellInfo(GetSpellInfo(51723))) then
				_,_,PriorityIcon = GetSpellInfo(51723)

			end


		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Kick if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(1766)) and UnitIsEnemy("target", "player") and (KickCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(1766)

		end
		
	end
	
------------------
-- ReminderIcon --
------------------

	-- Deadly Poison
	if(GetSpellInfo(GetSpellInfo(2823)) and not DeadlyPoisonUp) then
		_,_,ReminderIcon = GetSpellInfo(2823)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Preparation if Vanish isn't up and has more than 60 sec CD left
		if(GetSpellInfo(GetSpellInfo(14185)) and (VanishCD > 60) and not VanishUp and (PreparationCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(14185)

		-- Vanish if not vanished already and neither of Stealth or Shadow Blades are up, and the following Ambush will not waste Combo Points
		elseif(GetSpellInfo(GetSpellInfo(1856)) and (VanishCD <= GCD) and not StealthUp and not ShadowBladesUp and ((ComboPoints < 4) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 4)))) then
			_,_,ReminderIcon = GetSpellInfo(1856)

		-- Shadow Blades
		elseif(GetSpellInfo(GetSpellInfo(121471)) and (ShadowBladesCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(121471)

		end
		
	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon, ComboPoints, ResourceLow
	
end