function NextUpFunctions.RogueCombatPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local ResourceLow
	local CurrentTime = GetTime()
	local GCD = 1
	
	-- Get cooldowns
	local MarkedForDeathCD = NextUpFunctions.GetSpellCooldown(137619)
	local AdrenalineRushCD = NextUpFunctions.GetSpellCooldown(13750)
	local ShadowBladesCD = NextUpFunctions.GetSpellCooldown(121471)
	local KillingSpreeCD = NextUpFunctions.GetSpellCooldown(51690)
	local PreparationCD = NextUpFunctions.GetSpellCooldown(14185)
	local VanishCD = NextUpFunctions.GetSpellCooldown(1856)
	local KickCD = NextUpFunctions.GetSpellCooldown(1766)

	-- Get buffs
	local AdrenalineRushUp = UnitBuff("player", GetSpellInfo(13750))
	local ShadowBladesUp = UnitBuff("player", GetSpellInfo(121471))
	local DeadlyPoisonUp = UnitBuff("player", GetSpellInfo(2823))
	local StealthUp = UnitBuff("player", GetSpellInfo(1784))
	local VanishUp = UnitBuff("player", GetSpellInfo(1856))

	local _,_,_,AnticipationCharges = UnitBuff("player", GetSpellInfo(114015))
	if not AnticipationCharges then AnticipationCharges = 0 end
	
	local _,_,_,BanditsGuileStacks = UnitBuff("player", GetSpellInfo(84654))
	if not BanditsGuileStacks then BanditsGuileStacks = 0 end
	
	local SliceAndDiceUp,_,_,_,_,_,SliceAndDiceExpirationTime = UnitBuff("player", GetSpellInfo(5171))
	local SliceAndDiceRemaining = 0
	if SliceAndDiceUp then SliceAndDiceRemaining = (SliceAndDiceExpirationTime - CurrentTime) end
	
	local RuptureUp
	local RuptureRemaining = 0
	local _,_,_,_,_,_,RuptureExpirationTime,RuptureCaster = UnitDebuff("target", GetSpellInfo(1943))
	if (RuptureCaster == "player") then
		RuptureUp = true
		RuptureRemaining = (RuptureExpirationTime - CurrentTime)
	end
	
	local RevealingStrikeUp
	local RevealingStrikeRemaining = 0
	local _,_,_,_,_,_,RevealingStrikeExpirationTime,RevealingStrikeCaster = UnitDebuff("target", GetSpellInfo(84617))
	if (RevealingStrikeCaster == "player") then
		RevealingStrikeUp = true
		RevealingStrikeRemaining = (RevealingStrikeExpirationTime - CurrentTime)
	end
	
	local MainHandPoisonUp,_,_,OffHandPoisonUp = GetWeaponEnchantInfo()
	
	-- Get target debuffs
	local WeakenedArmorUp 
	if (UnitDebuff("target", GetSpellInfo(113746))) then
		WeakenedArmorUp = true
	end
	
	-- Get power levels
	local Energy = UnitPower("player", 3)
	local ComboPoints = GetComboPoints("player", "target")
	
	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Ambush if stealthed
			if(GetSpellInfo(GetSpellInfo(8676)) and (StealthUp or VanishUp)) then
				_,_,PriorityIcon = GetSpellInfo(8676)
				if Energy < 60 then ResourceLow = true end
				
			-- Slice and Dice if it's just about to wear off or isn't up, no matter how many combo points
			elseif(GetSpellInfo(GetSpellInfo(5171)) and (ComboPoints > 0) and (SliceAndDiceRemaining < 2)) then
				_,_,PriorityIcon = GetSpellInfo(5171)
				if Energy < 25 then ResourceLow = true end

			-- Marked for death if no Combo Points
			elseif(GetSpellInfo(GetSpellInfo(137619)) and (MarkedForDeathCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(137619)

			-- Revealing Strike unless it will cap Combo Points (Revealing Strike procs)
			elseif(GetSpellInfo(GetSpellInfo(84617)) and (RevealingStrikeRemaining < 2) and ((ComboPoints < 5) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 5))) and not RevealingStrikeUp) then
				_,_,PriorityIcon = GetSpellInfo(84617)
				if Energy < 40 then ResourceLow = true end

			-- Sinister Strike unless it will cap Combo Points (Revealing Strike procs)
			elseif(GetSpellInfo(GetSpellInfo(1752)) and ((ComboPoints < 5) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 5))) and not RevealingStrikeUp) then
				_,_,PriorityIcon = GetSpellInfo(1752)
				if Energy < 40 then ResourceLow = true end

			-- Rupture if it's just about to wear off or isn't up, no matter how many combo points
			elseif(GetSpellInfo(GetSpellInfo(1943)) and (ComboPoints > 0) and (RuptureRemaining < 2)) then
				_,_,PriorityIcon = GetSpellInfo(1943)
				if Energy < 25 then ResourceLow = true end

			-- Eviscerate with 5 Combo points
			elseif(GetSpellInfo(GetSpellInfo(2098)) and (ComboPoints >= 5)) then
				_,_,PriorityIcon = GetSpellInfo(2098)
				if Energy < 35 then ResourceLow = true end

			-- Keep Revealing Strike up
			elseif(GetSpellInfo(GetSpellInfo(84617)) and (RevealingStrikeRemaining < 2)) then
				_,_,PriorityIcon = GetSpellInfo(84617)
				if Energy < 40 then ResourceLow = true end

			-- Keep weakaned armor debuff up on target if noone else does
			elseif(GetSpellInfo(GetSpellInfo(8647)) and not WeakenedArmorUp) then
				_,_,PriorityIcon = GetSpellInfo(8647)
				if Energy < 20 then ResourceLow = true end

			-- Build Combo points with Sinister Strike
			elseif(GetSpellInfo(GetSpellInfo(1752))) then
				_,_,PriorityIcon = GetSpellInfo(1752)
				if Energy < 40 then ResourceLow = true end

			end

		else

			-- AoE priority --

			-- Crimson Tempest at 5 combo points
			if(GetSpellInfo(GetSpellInfo(121411)) and (ComboPoints == 5)) then
				_,_,PriorityIcon = GetSpellInfo(121411)
				if Energy < 40 then ResourceLow = true end

			-- Build combo points with Fan of Knives
			elseif(GetSpellInfo(GetSpellInfo(51723))) then
				_,_,PriorityIcon = GetSpellInfo(51723)

			end


		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Kick if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(1766)) and UnitIsEnemy("target", "player") and (KickCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(1766)

		end
	end
	
------------------
-- ReminderIcon --
------------------

	-- Deadly Poison
	if(GetSpellInfo(GetSpellInfo(2823)) and not DeadlyPoisonUp) then
		_,_,ReminderIcon = GetSpellInfo(2823)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Preparation if Vanish isn't up and has more than 60 sec CD left
		if(GetSpellInfo(GetSpellInfo(14185)) and (VanishCD > 60) and not VanishUp and (PreparationCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(14185)
		
		-- Vanish if not vanished already and neither of Stealth or Shadow Blades are up, and the following Ambush will not waste Combo Points. Energy threshhold ajusted for different Tier 1 talents
		elseif(
		GetSpellInfo(GetSpellInfo(1856)) and (VanishCD <= GCD) and not StealthUp
		and (((ComboPoints < 3) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 3))) or (not ShadowBladesUp and ((ComboPoints < 4) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 4)))))
		and ((GetSpellInfo(GetSpellInfo(108209)) and not AdrenalineRushUp and (Energy <20)) or (GetSpellInfo(GetSpellInfo(108209)) and (Energy >= 90)) or (not GetSpellInfo(GetSpellInfo(108209)) and not GetSpellInfo(GetSpellInfo(108209)) and (Energy >= 60)))
		) then
			_,_,ReminderIcon = GetSpellInfo(1856)

		-- Shadow Blades
		elseif(GetSpellInfo(GetSpellInfo(121471)) and (ShadowBladesCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(121471)

		-- Killing Spree if low on Energy and Adrenaline rush is not up
		elseif(GetSpellInfo(GetSpellInfo(51690)) and (KillingSpreeCD <= GCD) and (Energy < 35) and not AdrenalineRushUp) then
			_,_,ReminderIcon = GetSpellInfo(51690)

		-- Adrenaline Rush
		elseif(GetSpellInfo(GetSpellInfo(13750)) and (AdrenalineRushCD <= GCD) and ((Energy < 35) or ShadowBladesUp)) then
			_,_,ReminderIcon = GetSpellInfo(13750)

		end

	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon, ComboPoints, ResourceLow
end