function NextUpFunctions.RogueSubteltyPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local ResourceLow
	local CurrentTime = GetTime()
	local GCD = 1
	
	-- Get cooldowns
	local MarkedForDeathCD = NextUpFunctions.GetSpellCooldown(137619)
	local PremeditationCD = NextUpFunctions.GetSpellCooldown(14183)
	local ShurikenTossCD = NextUpFunctions.GetSpellCooldown(114014)
	local ShadowDanceCD = NextUpFunctions.GetSpellCooldown(51713)
	local PreparationCD = NextUpFunctions.GetSpellCooldown(14185)
	local VanishCD = NextUpFunctions.GetSpellCooldown(1856)
	local KickCD = NextUpFunctions.GetSpellCooldown(1766)

	-- Get buffs
	local MasterOfSubteltyUp = UnitBuff("player", GetSpellInfo(31223))
	local FindWeaknessUp = UnitDebuff("target", GetSpellInfo(91023))
	local HemorrhageUp = UnitDebuff("target", GetSpellInfo(16511))
	local DeadlyPoisonUp = UnitBuff("player", GetSpellInfo(2823))
	local ShadowDanceUp = UnitBuff("player", GetSpellInfo(51713))
	local StealthUp = UnitBuff("player", GetSpellInfo(1784))
	local VanishUp = UnitBuff("player", GetSpellInfo(1856))

	local SliceAndDiceUp,_,_,_,_,_,SliceAndDiceExpirationTime = UnitBuff("player", GetSpellInfo(5171))
	local SliceAndDiceRemaining = 0
	if SliceAndDiceUp then SliceAndDiceRemaining = (SliceAndDiceExpirationTime - CurrentTime) end
	
	local RuptureUp
	local RuptureRemaining = 0
	local _,_,_,_,_,_,RuptureExpirationTime,RuptureCaster = UnitDebuff("target", GetSpellInfo(1943))
	if (RuptureCaster == "player") then
		RuptureUp = true
		RuptureRemaining = (RuptureExpirationTime - CurrentTime)
	end
	
	local MainHandPoisonUp,_,_,OffHandPoisonUp = GetWeaponEnchantInfo()
	
	-- Get target debuffs
	local WeakenedArmorUp 
	if (UnitDebuff("target", GetSpellInfo(113746))) then
		WeakenedArmorUp = true
	end
	
	-- Get power levels
	local Energy = UnitPower("player", 3)
	local ComboPoints = GetComboPoints("player", "target")
	
	-- Get target's health and spells being cast
	local TargetHP = UnitHealth("target")
	local TargetHPMax = UnitHealthMax("target")
	local TargetHPPercent = TargetHP / TargetHPMax * 100

------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Ambush if stealthed (or shadow dance is up allowing stealth abilities) and Combo Points are below 5 or Anticipation Charges are below 4
			if(GetSpellInfo(GetSpellInfo(8676)) and (StealthUp or VanishUp or ShadowDanceUp) and ((ComboPoints < 5) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 4)))) then
				_,_,PriorityIcon = GetSpellInfo(8676)
				if Energy < 60 then ResourceLow = true end

			-- Pool up 75 energy if Shadow Dance is available and player is not in stealth
			elseif(GetSpellInfo(GetSpellInfo(51713)) and (ShadowDanceCD <= GCD) and not StealthUp and not VanishUp) then
				if Energy > 75 then _,_,ReminderIcon = GetSpellInfo(51713) end

			-- Pool up 45 energy if Vanish is available, Combo Points are below 4 and Shadow Dance, Master of Subtelty and Find Weakness are all down
			elseif(GetSpellInfo(GetSpellInfo(1856)) and (VanishCD <= GCD) and (ComboPoints < 4) and not ShadowDanceUp and not MasterOfSubteltyUp and not FindWeaknessUp) then
				if Energy > 45 then _,_,ReminderIcon = GetSpellInfo(1856) end

			-- Marked for death if no Combo Points
			elseif(GetSpellInfo(GetSpellInfo(137619)) and (MarkedForDeathCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(137619)

			-- 5 Combo Point Slice and Dice if it has 4 seconds or less remaining
			elseif(GetSpellInfo(GetSpellInfo(5171)) and (ComboPoints >= 5) and (SliceAndDiceRemaining < 4)) then
				_,_,PriorityIcon = GetSpellInfo(5171)
				if Energy < 25 then ResourceLow = true end

			-- 5 Combo Point Rupture if it has 3 seconds or less remaining
			elseif(GetSpellInfo(GetSpellInfo(1943)) and (ComboPoints >= 5) and (RuptureRemaining < 3)) then
				_,_,PriorityIcon = GetSpellInfo(1943)
				if Energy < 25 then ResourceLow = true end

			-- 5 Combo point Eviscerate
			elseif(GetSpellInfo(GetSpellInfo(2098)) and (ComboPoints == 5)) then
				_,_,PriorityIcon = GetSpellInfo(2098)
				if Energy < 35 then ResourceLow = true end

			-- Keep Hemorrhage bleed up
			elseif(GetSpellInfo(GetSpellInfo(16511)) and not HemorrhageUp) then
				_,_,PriorityIcon = GetSpellInfo(16511)
				if Energy < 30 then ResourceLow = true end

			-- Shuriken Toss if Energy is below 65
			elseif(GetSpellInfo(GetSpellInfo(114014)) and (ShurikenTossCD <= GCD) and (Energy < 65)) then
				_,_,PriorityIcon = GetSpellInfo(114014)
				if Energy < 30 then ResourceLow = true end

			-- Keep weakaned armor debuff up on target if noone else does
			elseif(GetSpellInfo(GetSpellInfo(8647)) and not WeakenedArmorUp) then
				_,_,PriorityIcon = GetSpellInfo(8647)
				if Energy < 20 then ResourceLow = true end

			-- Preparation if Vanish isn't up and has more than 60 sec CD left
			elseif(GetSpellInfo(GetSpellInfo(14185)) and (VanishCD > 60) and not VanishUp and (Energy < 35) and (PreparationCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(14185)

			-- Build Combo points with Backstab
			elseif(GetSpellInfo(GetSpellInfo(53))) then
				_,_,PriorityIcon = GetSpellInfo(53)
				if Energy < 35 then ResourceLow = true end

			end

		else

			-- AoE priority --

			-- Crimson Tempest at 5 combo points
			if(GetSpellInfo(GetSpellInfo(121411)) and (ComboPoints == 5)) then
				_,_,PriorityIcon = GetSpellInfo(121411)
				if Energy < 35 then ResourceLow = true end

			-- Build combo points with Fan of Knives
			elseif(GetSpellInfo(GetSpellInfo(51723))) then
				_,_,PriorityIcon = GetSpellInfo(51723)

			end


		end
		
-------------------
-- InterruptIcon --
-------------------

		-- Kick if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(1766)) and UnitIsEnemy("target", "player") and (KickCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(1766)

		end
	end
	
------------------
-- ReminderIcon --
------------------

	-- Deadly Poison
	if(GetSpellInfo(GetSpellInfo(2823)) and not DeadlyPoisonUp) then
		_,_,ReminderIcon = GetSpellInfo(2823)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Shadow Blades
		if(GetSpellInfo(GetSpellInfo(121471)) and (ShadowBladesCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(121471)

		-- Premeditation if stealthed and it will not waste Combo Points
		elseif(GetSpellInfo(GetSpellInfo(14183)) and (StealthUp or VanishUp or ShadowDanceUp) and ((ComboPoints < 4) or (GetSpellInfo(GetSpellInfo(114015)) and (AnticipationCharges < 4))) and (PremeditationCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(14183)

		end
	
	end

	return PriorityIcon, InterruptIcon, ReminderIcon, ComboPoints, ResourceLow
end