function NextUpFunctions.ShamanElementalPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(324) -- Get Lightning Shield CD

	-- Get cooldowns
	local AncestralSwiftnessCD = NextUpFunctions.GetSpellCooldown(16188)
	local FireElementalTotemCD = NextUpFunctions.GetSpellCooldown(2894)
	local ElementalMasteryCD = NextUpFunctions.GetSpellCooldown(16166)
	local UnleashElementsCD = NextUpFunctions.GetSpellCooldown(73680)
	local StormlashTotemCD = NextUpFunctions.GetSpellCooldown(120668)
	local ElementalBlastCD = NextUpFunctions.GetSpellCooldown(117014)
	local AscendanceCD = NextUpFunctions.GetSpellCooldown(114049)
	local EarthquakeCD = NextUpFunctions.GetSpellCooldown(61882)
	local FlameShockCD = NextUpFunctions.GetSpellCooldown(8050)
	local EarthShockCD = NextUpFunctions.GetSpellCooldown(8042)
	local WindShearCD = NextUpFunctions.GetSpellCooldown(57994)
	local LavaBurstCD = NextUpFunctions.GetSpellCooldown(51505)


	-- Get buffs and debuffs
	local StormlashTotemUp = UnitBuff("player", GetSpellInfo(120668))
	local AscendanceUp = UnitBuff("player", GetSpellInfo(114049))
	local LavaSurgeUp = UnitBuff("player", GetSpellInfo(77756))
	local FlametongueUp = GetWeaponEnchantInfo()

	local LightningShieldUp,_,_,LightningShieldCharges = UnitBuff("player", GetSpellInfo(324))
	if not LightningShieldCharges then LightningShieldCharges = 0 end

	local FlameShockUp
	local FlameShockRemaining = 0
	local _,_,_,_,_,_,FlameShockExpirationTime,FlameShockCaster = UnitDebuff("target", GetSpellInfo(8050))
	if (FlameShockCaster == "player") then
		FlameShockUp = true
		FlameShockRemaining = (FlameShockExpirationTime - CurrentTime)
	end
	
	-- Get totem info. GetTotemInfo(1) returns data for any fire totem: Searing, Magma or Elemental.
	local _,_,FireTotemStartTime,FireTotemDuration = GetTotemInfo(1)
	local FireTotemRemaining = max((FireTotemStartTime + FireTotemDuration - CurrentTime), 0)
		
------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Unleash Elements if specced into Unleashed Fury
			if(GetSpellInfo(GetSpellInfo(117012)) and not AscendanceUp and UnleashElementsCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(73680)

			-- Lava Burst on cooldown
			elseif(GetSpellInfo(GetSpellInfo(51505)) and (((UnitCastingInfo("player") ~= GetSpellInfo(51505)) and (LavaBurstCD <= GCD)) or AscendanceUp) and (FlameShockRemaining > GCD)) then
				_,_,PriorityIcon = GetSpellInfo(51505)

			-- Flame Shock if it's about to expire
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and (FlameShockRemaining < 3)) then
				_,_,PriorityIcon = GetSpellInfo(8050)
			
			-- Elemental Blast if specced
			elseif(GetSpellInfo(GetSpellInfo(117014)) and (UnitCastingInfo("player") ~= GetSpellInfo(117014)) and ElementalBlastCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(117014)

			-- Earth Shock at 7 Ligtning Shield charges, or as few as three to time the updating of Flame Shock
			elseif(GetSpellInfo(GetSpellInfo(8042)) and (EarthShockCD <= GCD) and ((LightningShieldCharges == 7) or ((LightningShieldCharges >= 3) and ((FlameShockRemaining < 8)and(FlameShockRemaining > 5))))) then
				_,_,PriorityIcon = GetSpellInfo(8042)

			-- Flame Shock if Ascendance is off CD and Flame Shock's duration won't last for the whole Ascendance duration
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and (FlameShockRemaining < 15) and ((AscendanceCD + 15) < 30)) then
				_,_,PriorityIcon = GetSpellInfo(8050)

			-- Keep Searing Totem up
			elseif(GetSpellInfo(GetSpellInfo(3599)) and (FireTotemRemaining <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(3599)

			-- Lightning Bolt
			elseif(GetSpellInfo(GetSpellInfo(403))) then
				_,_,PriorityIcon = GetSpellInfo(403)

			end

		else

			-- AoE priority --

			-- Earthquake
			if(GetSpellInfo(GetSpellInfo(61882) and (UnitCastingInfo("player") ~= GetSpellInfo(61882))) and EarthquakeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(61882)

			-- Chain Lightning
			elseif(GetSpellInfo(GetSpellInfo(421) and (UnitCastingInfo("player") ~= GetSpellInfo(421)))) then
				_,_,PriorityIcon = GetSpellInfo(421)
			
			end

		end	
			
-------------------
-- InterruptIcon --
-------------------

		-- Wind Shear if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(57994)) and (WindShearCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(57994)
		end
	end
	
------------------
-- ReminderIcon --
------------------

	-- Flametounge Weapon
	if(GetSpellInfo(GetSpellInfo(8024)) and not FlametongueUp) then
		_,_,ReminderIcon = GetSpellInfo(8024)

	-- Lightning Shield
	elseif(GetSpellInfo(GetSpellInfo(324)) and not LightningShieldUp) then
		_,_,ReminderIcon = GetSpellInfo(324)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Stormlash Totem if Heroism is affecting the player
		if(GetSpellInfo(GetSpellInfo(120668)) and (StormlashTotemCD <= GCD) and (NextUpFunctions.BurstHasteUp()) and not StormlashTotemUp) then
			_,_,ReminderIcon = GetSpellInfo(120668)

		-- Elemental Mastery if no other burst haste buffs are on the player, such as a troll's Berserking or Heroism. Used in conjunction with Ascendance if the player has it.
		elseif(GetSpellInfo(GetSpellInfo(16166)) and (ElementalMasteryCD <= GCD) and not NextUpFunctions.BurstHasteUp() and not UnitBuff("player", GetSpellInfo(26297))
		and ((GetSpellInfo(GetSpellInfo(114049)) and AscendanceUp) or (not GetSpellInfo(GetSpellInfo(114049))))) then
			_,_,ReminderIcon = GetSpellInfo(16166)

		-- Ancestral Swiftness if Ascendance buff isn't up
		elseif(GetSpellInfo(GetSpellInfo(16188)) and (AncestralSwiftnessCD <= GCD) and not AscendanceUp) then
			_,_,ReminderIcon = GetSpellInfo(16188)

		-- Fire Elemental Totem
		elseif(GetSpellInfo(GetSpellInfo(2894)) and (FireElementalTotemCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(2894)

		-- Ascendance if Flame Shock won't need refreshing during Ascendance's duration and Lava Burst is on CD
		elseif(GetSpellInfo(GetSpellInfo(114049)) and (AscendanceCD <= GCD) and not AscendanceUp and (FlameShockRemaining > 15) and (GetSpellCooldown(51505) ~= 0)) then
			_,_,ReminderIcon = GetSpellInfo(114049)

		end

	end
	
	return PriorityIcon, InterruptIcon, ReminderIcon, LightningShieldCharges-2
end