function NextUpFunctions.ShamanEnhancementPriority(SingleTargetPriority)
	local PriorityIcon
	local InterruptIcon
	local ReminderIcon
	local ResourceLow
	local CurrentTime = GetTime()
	local GCD = NextUpFunctions.GetGCD(324) -- Get Lightning Shield CD
	
	-- Get cooldowns
	local AncestralSwiftnessCD = NextUpFunctions.GetSpellCooldown(16188)
	local FireElementalTotemCD = NextUpFunctions.GetSpellCooldown(2894)
	local UnleashElementsCD = NextUpFunctions.GetSpellCooldown(73680)
	local ElementalBlastCD = NextUpFunctions.GetSpellCooldown(117014)
	local StormlashTotemCD = NextUpFunctions.GetSpellCooldown(120668)
	local StormstrikeCD = NextUpFunctions.GetSpellCooldown(17364)
	local FeralSpiritCD = NextUpFunctions.GetSpellCooldown(51533)
	local StormblastCD = NextUpFunctions.GetSpellCooldown(115356)
	local AscendanceCD = NextUpFunctions.GetSpellCooldown(114049)
	local FlameShockCD = NextUpFunctions.GetSpellCooldown(8050)
	local EarthShockCD = NextUpFunctions.GetSpellCooldown(8042)
	local WindShearCD = NextUpFunctions.GetSpellCooldown(57994)
	local LavaLashCD = NextUpFunctions.GetSpellCooldown(60103)
	local FireNovaCD = NextUpFunctions.GetSpellCooldown(1535)
	
	-- Get buffs and debuffs
	local AncestralSwiftnessUp = UnitBuff("player", GetSpellInfo(16188))
	local StormlashTotemUp = UnitBuff("player", GetSpellInfo(120668))
	local LightningShieldUp = UnitBuff("player", GetSpellInfo(324))
	local AscendanceUp = UnitBuff("player", GetSpellInfo(114049))
	local WindfuryUp,_,_,FlametongueUp = GetWeaponEnchantInfo()

	local _,_,_,MaelstromStacks = UnitBuff("player", GetSpellInfo(51530))
	if not MaelstromStacks then MaelstromStacks = 0 end

	local UnleashedFuryUp
	local _,_,_,_,_,_,_,UnleashedFuryCaster = UnitDebuff("target", GetSpellInfo(118470))
	if (UnleashedFuryCaster == "player") then
		UnleashedFuryUp = true
	end

	local UnleashFlameUp
	local UnleashFlameRemaining = 0
	local _,_,_,_,_,_,UnleashFlameExpirationTime = UnitBuff("player", GetSpellInfo(73683))
	if UnleashFlameExpirationTime then
		UnleashFlameUp = true
		UnleashFlameRemaining = (UnleashFlameExpirationTime - CurrentTime)
	end

	local FlameShockUp
	local FlameShockRemaining = 0
	local _,_,_,_,_,_,FlameShockExpirationTime,FlameShockCaster = UnitDebuff("target", GetSpellInfo(8050))
	if (FlameShockCaster == "player") then
		FlameShockUp = true
		FlameShockRemaining = (FlameShockExpirationTime - CurrentTime)
	end
	
	-- Get totem info. GetTotemInfo(1) returns data for any fire totem: Searing, Magma or Elemental.
	local _,_,FireTotemStartTime,FireTotemDuration = GetTotemInfo(1)
	local FireTotemRemaining = max((FireTotemStartTime + FireTotemDuration - CurrentTime), 0)
		
------------------
-- PriorityIcon --
------------------

	if NextUpFunctions.TargetIsEnemy() then

		if SingleTargetPriority then
		
			-- Single target priority --

			-- Fire Totem when it expires
			if(GetSpellInfo(GetSpellInfo(3599)) and (FireTotemRemaining <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(3599)

			-- Unleash Elements if specced into Unleashed Fury
			elseif(GetSpellInfo(GetSpellInfo(117012)) and UnleashElementsCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(73680)

			-- Elemental Blast if specced
			elseif(GetSpellInfo(GetSpellInfo(117014)) and (UnitCastingInfo("player") ~= GetSpellInfo(117014)) and ElementalBlastCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(117014)

			-- Lightning Bolt at 5 Maelstrom Weapon charges
			elseif(GetSpellInfo(GetSpellInfo(403)) and MaelstromStacks == 5) then
				_,_,PriorityIcon = GetSpellInfo(403)

			-- Stormblast (Ascendance Stormstrike)
			elseif(GetSpellInfo(GetSpellInfo(115356)) and AscendanceUp and StormblastCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(115356)

			-- Stormstrike
			elseif(GetSpellInfo(GetSpellInfo(17364)) and StormstrikeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(17364)

			-- Flame Shock if it's expired and Unleash Flame buff is up
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and (not FlameShockUp or UnleashFlameUp)) then
				_,_,PriorityIcon = GetSpellInfo(8050)

			--	Lava Lash
			elseif(GetSpellInfo(GetSpellInfo(60103)) and (LavaLashCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(60103)

			-- Flame Shock if Unleash Flame buff is up
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and UnleashFlameUp) then
				_,_,PriorityIcon = GetSpellInfo(8050)

			-- Unleash Elements
			elseif(GetSpellInfo(GetSpellInfo(73680)) and UnleashElementsCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(73680)

			-- Lightning bolt if Maelstromstacks >= 3, Unleashed Fury is on target and Ascendance isn't up
			elseif(GetSpellInfo(GetSpellInfo(403)) and (MaelstromStacks >= 3) and UnleashedFuryUp and (UnitCastingInfo("player") ~= GetSpellInfo(403)) and not AscendanceUp) then
				_,_,PriorityIcon = GetSpellInfo(403)

			-- Ancestral Swiftness if specced and Maelstrom Weapon charges are below 2
			elseif(GetSpellInfo(GetSpellInfo(16188)) and (AncestralSwiftnessCD <= GCD) and (MaelstromWeaponCharges < 2)) then
				_,_,PriorityIcon = GetSpellInfo(16188)

			-- Lightning Bolt if Ancestral Swiftness is up
			elseif(GetSpellInfo(GetSpellInfo(403)) and AncestralSwiftnessUp) then
				_,_,PriorityIcon = GetSpellInfo(403)

			-- Earth Shock
			elseif(GetSpellInfo(GetSpellInfo(8042)) and (EarthShockCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(8042)

			-- Feral Spirit
			elseif(GetSpellInfo(GetSpellInfo(51533)) and (FeralSpiritCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(51533)

			end

			-- If no spell can be cast within one GCD, show the spell coming off cooldown first
			if not PriorityIcon then
				ResourceLow = true -- Enables the dark overlay on the Priority Icon to indicate that the spell showing is still on CD
				local SpellCooldownArray = {}
				for i = 1, 7 do
					SpellCooldownArray[i] = {}
				end
				
				SpellCooldownArray[1][1] = 51533 -- Feral Spirit
				SpellCooldownArray[1][2] = FeralSpiritCD

				SpellCooldownArray[2][1] = 8042 -- Earth Shock
				SpellCooldownArray[2][2] = FlameShockCD

				SpellCooldownArray[3][1] = 73680 -- Unleash Elements
				SpellCooldownArray[3][2] = UnleashElementsCD

				SpellCooldownArray[4][1] = 60103 -- Lava Lash
				SpellCooldownArray[4][2] = LavaLashCD

				SpellCooldownArray[5][1] = 17364 -- Stormstrike
				SpellCooldownArray[5][2] = StormstrikeCD

				if GetSpellInfo(GetSpellInfo(117014)) then
				SpellCooldownArray[6][1] = 117014 -- Elemental Blast, only added to the array if specced
				SpellCooldownArray[6][2] = ElementalBlastCD
				end
				
				SpellCooldownArray[7][1] = 3599 -- Fire Totem
				SpellCooldownArray[7][2] = FireTotemRemaining

				local ShortestCD = FeralSpiritCD -- Value must be higher than or equal to the shortest cooldown

				for i = 1, 7 do
					if ((UnitCastingInfo("player") ~= GetSpellInfo(SpellCooldownArray[i][1])) and GetSpellInfo(SpellCooldownArray[i][1])) then -- If the current spell in the Array exists and is not the one currently being cast...
						if (SpellCooldownArray[i][2] <= ShortestCD) then -- ...check if spells' CD is shorter than the shortest found so far
							ShortestCD = SpellCooldownArray[i][2] -- If so, set ShortestCD to the current spell's CD...
							_,_,PriorityIcon = GetSpellInfo(SpellCooldownArray[i][1]) -- ...and change the icon appropriately
						end
					end
				end
				
				-- Swap Earth Shock icon for Flame Shock if Unleash Flame is up or Flame Shock DOT is not up when the Shocks come off cooldown
				if ((PriorityIcon == (select(3,GetSpellInfo(GetSpellInfo(8042))))) and ((FlameShockRemaining < ShortestCD) or (UnleashFlameRemaining > ShortestCD))) then _,_,PriorityIcon = GetSpellInfo(GetSpellInfo(8050)) end

				-- Remove the dark overlay if Fire Totem is the next spell available. Refreshing it early is not bad, if there are no other spells to cast instead.
				if (PriorityIcon == (select(3,GetSpellInfo(GetSpellInfo(3599))))) then ResourceLow = false end
			end
			
		else

			-- AoE priority --

			-- Fire Nova
			if(GetSpellInfo(GetSpellInfo(1535)) and FlameShockUp and FireNovaCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(1535)

			-- Flame Shock if it can be spread with Lava Lash
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and (LavaLashCD <= 2*GCD)) then
				_,_,PriorityIcon = GetSpellInfo(8050)

			-- 	Lava Lash to spread Flame Shock
			elseif(GetSpellInfo(GetSpellInfo(60103)) and FlameShockUp and (LavaLashCD <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(60103)

			-- Magma Totem when it expires
			elseif(GetSpellInfo(GetSpellInfo(8190)) and (FireTotemRemaining <= GCD)) then
				_,_,PriorityIcon = GetSpellInfo(8190)

			-- Unleash Elements if specced into Unleashed Fury
			elseif(GetSpellInfo(GetSpellInfo(117012)) and UnleashElementsCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(73680)

			-- Elemental Blast if specced
			elseif(GetSpellInfo(GetSpellInfo(117014)) and (UnitCastingInfo("player") ~= GetSpellInfo(117014)) and ElementalBlastCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(117014)

			-- Chain Lightning at 5 Maelstrom Weapon charges
			elseif(GetSpellInfo(GetSpellInfo(421)) and MaelstromStacks == 5) then
				_,_,PriorityIcon = GetSpellInfo(421)

			-- Stormblast (Ascendance Stormstrike)
			elseif(GetSpellInfo(GetSpellInfo(115356)) and AscendanceUp and StormblastCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(115356)

			-- Flame Shock if it's about to expire or Unleash Flame buff is present
			elseif(GetSpellInfo(GetSpellInfo(8050)) and (FlameShockCD <= GCD) and (not FlameShockUp or UnleashFlameUp)) then
				_,_,PriorityIcon = GetSpellInfo(8050)

			-- Stormstrike
			elseif(GetSpellInfo(GetSpellInfo(17364)) and StormstrikeCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(17364)

			-- Unleash Elements
			elseif(GetSpellInfo(GetSpellInfo(73680)) and UnleashElementsCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(73680)

			-- Chain Lightning if Maelstromstacks >= 3, Unleashed Fury is on target and Ascendance isn't up
			elseif(GetSpellInfo(GetSpellInfo(421)) and (MaelstromStacks >= 3) and (UnitCastingInfo("player") ~= GetSpellInfo(421)) and UnleashedFuryUp and not AscendanceUp) then
				_,_,PriorityIcon = GetSpellInfo(421)

			-- Earth Shock
			elseif(GetSpellInfo(GetSpellInfo(8042)) and EarthShockCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(8042)

			-- Feral Spirit
			elseif(GetSpellInfo(GetSpellInfo(51533)) and FeralSpiritCD <= GCD) then
				_,_,PriorityIcon = GetSpellInfo(51533)
			end
		end	
			
-------------------
-- InterruptIcon --
-------------------

		-- Wind Shear if target is casting an interruptable spell
		if (GetSpellInfo(GetSpellInfo(57994)) and (WindShearCD == 0) and NextUpFunctions.TargetSpellcastIsInterruptable()) then
			_,_,InterruptIcon = GetSpellInfo(57994)
		end

	end
	
------------------
-- ReminderIcon --
------------------

	-- Windfury Weapon
	if(GetSpellInfo(GetSpellInfo(8232)) and not WindfuryUp) then
		_,_,ReminderIcon = GetSpellInfo(8232)

	-- Flametounge Weapon
	elseif(GetSpellInfo(GetSpellInfo(8024)) and not FlametongueUp) then
		_,_,ReminderIcon = GetSpellInfo(8024)

	-- Lightning Shield
	elseif(GetSpellInfo(GetSpellInfo(324)) and not LightningShieldUp) then
		_,_,ReminderIcon = GetSpellInfo(324)

	-- Combat reminders start here
	elseif NextUpFunctions.TargetIsEnemy() then

		-- Stormlash Totem if Heroism or another burst haste effect is on the player
		if(GetSpellInfo(GetSpellInfo(120668)) and (StormlashTotemCD <= GCD) and (NextUpFunctions.BurstHasteUp()) and not StormlashTotemUp) then
			_,_,ReminderIcon = GetSpellInfo(120668)

		-- Fire Elemental Totem
		elseif(GetSpellInfo(GetSpellInfo(2894)) and (FireElementalTotemCD <= GCD)) then
			_,_,ReminderIcon = GetSpellInfo(2894)

		-- Ascendance
		elseif(GetSpellInfo(GetSpellInfo(114049)) and (AscendanceCD <= GCD) and not AscendanceUp) then
			_,_,ReminderIcon = GetSpellInfo(114049)

		end

	end
		
	return PriorityIcon, InterruptIcon, ReminderIcon, MaelstromStacks, ResourceLow
end