-- This .lua contains functions that are shared between the specs' separate .lua files

NextUpFunctions = {}

------------------------
-- Cooldown Functions --
------------------------

-- Returns the cooldown of a class-specific skill without cooldown available at level 10, or the remaining cast time if a spell is currently being cast. This is the global cooldown.
function NextUpFunctions.GetGCD(SpellID)
	return max((GetSpellCooldown(SpellID) + select(2,GetSpellCooldown(SpellID)) - GetTime()), (((select(6,UnitCastingInfo("player"))) or 0) / 1000 - GetTime()), 0)
end

-- Returns the cooldown remaining on the SpellID sent as argument
function NextUpFunctions.GetSpellCooldown(SpellID)
	return max((GetSpellCooldown(SpellID) + select(2,GetSpellCooldown(SpellID)) - GetTime()), 0)
end

-- Returns the cooldown remaining on the ItemID sent as argument
function NextUpFunctions.GetItemCooldown(ItemID)
	return max((GetItemCooldown(ItemID) + select(2,GetItemCooldown(ItemID)) - GetTime()), 0)
end

----------------------
-- Target Functions --
----------------------

-- Returns true if the current target is attackable (Enemy, friend under mind control effect or friend under Ambershaper Un'sok's Reshape Life effect)
function NextUpFunctions.TargetIsEnemy()
	return (UnitExists("target") and not UnitIsDead("target") and (not UnitIsFriend("player", "target") or (UnitIsFriend("player", "target") and  UnitIsCharmed("target")) or (UnitIsFriend("player","target") and UnitDebuff("target", GetSpellInfo(122370)))))
end

-- Returns true if the target is casting a spell that is interrulptible with Kick, Mind Freeze, Rebuke, Wind Shear, Skull Bash etc.
function NextUpFunctions.TargetSpellcastIsInterruptable()
	return (select(9, UnitCastingInfo("target")) == false)
end

---------------------------
-- HP and Mana Functions --
---------------------------

-- Returns the HP Percentage of the current target
function NextUpFunctions.GetTargetHPPercent()
	return (UnitHealth("target") / UnitHealthMax("target") * 100)
end

-- Returns the HP Percentage of the player
function NextUpFunctions.GetPlayerHPPercent()
	return (UnitHealth("player") / UnitHealthMax("player") * 100)
end

-- Returns the Mana Percentage of the player
function NextUpFunctions.GetPlayerManaPercent()
	return (UnitPower("player", 0) / UnitPowerMax("player", 0) * 100)
end

--------------------------
-- Buff Check Functions --
--------------------------

-- Returns true if any Spellpower Buff is up on the player
function NextUpFunctions.SpellPowerBuffUp()
	return (UnitBuff("player", GetSpellInfo(1459)) or UnitBuff("player", GetSpellInfo(61316)) or UnitBuff("player", GetSpellInfo(77747)) or UnitBuff("player", GetSpellInfo(109773)) or UnitBuff("player", GetSpellInfo(126309)))
end

-- Returns true if any Crit Buff is up on the player
function NextUpFunctions.CritBuffUp()
	return (UnitBuff("player", GetSpellInfo(1459)) or UnitBuff("player", GetSpellInfo(61316)) or UnitBuff("player", GetSpellInfo(17007)) or UnitBuff("player", GetSpellInfo(116781)) or UnitBuff("player", GetSpellInfo(97229)) or UnitBuff("player", GetSpellInfo(24604)) or UnitBuff("player", GetSpellInfo(90309)) or UnitBuff("player", GetSpellInfo(126373)) or UnitBuff("player", GetSpellInfo(126309)))
end

-- Returns true if any Stat Buff is up on the player (Mark of the Wild, Blessing of Kings etc.)
function NextUpFunctions.StatBuffUp()
	return (UnitBuff("player", GetSpellInfo(1126)) or UnitBuff("player", GetSpellInfo(20217)) or UnitBuff("player", GetSpellInfo(115921)) or UnitBuff("player", GetSpellInfo(90363)))
end

-- Returns true if any Mastery Buff is up on the player
function NextUpFunctions.MasteryBuffUp()
	return (UnitBuff("player", GetSpellInfo(19740)) or UnitBuff("player", GetSpellInfo(116956)) or UnitBuff("player", GetSpellInfo(93435)) or UnitBuff("player", GetSpellInfo(128997)))
end

-- Returns true if any Burst Haste Buff is up on the player (Heroism, Time Warp etc.)
function NextUpFunctions.BurstHasteUp()
	return (UnitBuff("player", GetSpellInfo(80353)) or UnitBuff("player", GetSpellInfo(2825)) or UnitBuff("player", GetSpellInfo(32182)) or UnitBuff("player", GetSpellInfo(90355)))
end